/*
    CalculatorSpin.cc

    This file contains two custom spinBox controls. Both controls
    are designed to call a pop-up calculator similar to the Windows
    Calculator.  

    CalculatorSpin calls a simple calculator that offers basic 
    arithmetic operations.

    CalculatorSpin2 calls an advanced calculator that incudes a 
    memory stack and non decimal calculations.

    Form metrics must be pixels.
    
    Authors: Michael Nuwer  (December 2003)
             Based on work by Paul Van House
             (March 2001)

    Dependencies:
       vStruct.prg
       vStruct.h
       Calculat.bmp



*/


CLASS CalculatorSpin2(parentObj) OF CalculatorSpin(parentObj) custom
   this.formname = "CALCULATOR2"
ENDCLASS


CLASS CalculatorSpin(parentObj) OF SpinBox(parentObj) 
	With (this)
      onOpen = class::CALCSPIN_ONOPEN
      Value = 0
      RangeReq = False
   EndWith

   function CALCSPIN_onOpen
      this.formatButton()
      return

   function formatButton
      privat cBtnName, c, cn  //, click

      //  the button name is PBDatePickerX....

      cBtnName = 'PB'+trim(This.Name)
      c = 'this.parent.'+cBtnName
      cn = c+' = new PUSHBUTTON(this.parent)'
      &cn
      cn = c+'.owner = this'
      &cn
      with (&c)
         group    = true
         Top      = this.top + 2
         Left     = this.left+this.width - 42
         Height   = this.height - 3
         Width    = 22  
         Text     = ""
         OnClick  = CLASS::PBCalculator_OnClick
         PageNo   = this.PageNo
         UpBitmap = "filename:2 Calculat.bmp"
         tabStop  = false
      endwith
      return

   function PBCalculator_OnClick

      local oChild, oRect, parentLeft, parentTop, ;
          parentWidth, parentHeight, nTitleBar
      private RtnVal //So we can determine "type" of returned value

      if type("this.owner.formname") # "U"
         if this.owner.formname == "CALCULATOR2"
            oChild = New Calculator2Form()
            oChild.lLink = True
            oChild.height = 200
         elseif this.owner.formname == "CALCULATOR1"
            oChild = New Calculator1Form()
         endif
      else
         //  Use Calc-1 for default
         oChild = New Calculator1Form()
      endif 
      oChild.mdi = False

      if This.Parent.Metric = 6
         //controls form is in pixels - position calculator near spinbox
         If Type("GetWindowRect") # "FP"
            extern CLOGICAL GetWindowRect(CHANDLE, CPTR) User32
         EndIf

         set procedure to :dUFLP:vStruct.prg additive
         oRect = new VStruct( 8*2 )
         GetWindowRect(this.parent.hwnd, oRect )

         parentLeft   = oRect.getLong( 0, false )
         parentTop    = oRect.getLong( 4, false )
         parentWidth  = oRect.getLong( 8, false )
         parentHeight = oRect.getLong( 12, false )

         close procedure :dUFLP:vStruct.prg
         if this.parent.baseClassName == "FORM"
            nTitleBar = 20
         else
            nTitleBar = 0
         endif 
         oChild.top  = parentTop + this.owner.top + this.owner.height ;
                         + nTitleBar + 3
         oChild.left = parentLeft + this.owner.left + this.owner.width ;
                         - oChild.width - 3
         // oChild.autoCenter = false

      EndIf 

      oChild.AutoCenter = (This.Parent.Metric <> 6)

      //Save "Set Decimals" Setting for restoration later.
      oChild.Save_Decimals =  Setto('Decimals')

      //Save "Set CuaEnter" Setting for restoration later
      //Must be ON in order for enter key to double as "equal" key
      oChild.Save_CuaEnter =  Set('CuaEnter')
      Set CUAEnter On

#define padl(cSTRING,nLEN,cPADCHAR) ;
        Transform(Replicate(cPADCHAR,nLEN - Len(Ltrim(Rtrim(cSTRING)))) + ;
        ltrim(rtrim(cSTRING)),replicate('X',nLEN))

      //Set calculator's display to equal spinbox value
      oChild.Display.Text = Padl(Class::Ztrim(this.owner.value),18," ")


      //Set other calculator variables to allow computations on initial value
      If "." $ oChild.Display.Text
         oChild.BeforePoint = False
         //DecPlaces variable to # of decimal places + 2
         oChild.DecPlaces = 18 -  at('.',oChild.Display.Text) + 2
         oChild.MostDecPlaces = oChild.DecPlaces
         set Decimals to oChild.DecPlaces
      Else
         oChild.BeforePoint = True
         //DecPlaces variable to 1
         oChild.DecPlaces = 1
         oChild.MostDecPlaces = 1
         Set Decimals to 1
      EndIf // "." $ CalculatorForm.Display.Text

      //Open the Calculator
      Rtn_Val = oChild.readmodal()
      //If closed with the paste button or Ctrl-V, and a
      //valid decimal number is in the result pane, a
      //number will be returned.
      //all other ways of closing the form return a logical True or False
      If Type('Rtn_Val') = "N"
      	//Closed with a valid number passed
         this.owner.value = Rtn_Val
         if .not. empty(this.owner.onChange)
             this.onChange()
         endif
      EndIf
      this.setfocus()

   Function Ztrim(nNumber,Decimals)
       Local X,Whole,fraction,To_Left
       If Pcount() < 2
         Decimals = 8
       EndIf
    	 Whole = int(nNumber)
       Fraction = nNumber - Whole
       ToLeft = Len(Ltrim(str(Whole)))

       X = Transform((Whole + Fraction),;
       		"@b " + replicate("9",ToLeft) + "." + replicate("9",Decimals))
       Do While Right(X,1) = '0'
         X = left(X,Len(x) - 1)
       EndDo
       If Right(X,1) = '.'
         X = left(X,Len(X) - 1)
       EndIf
       Return X

ENDCLASS




#define BASE_HEX             16
#define BASE_DEC             10
#define BASE_OCT              8
#define BASE_BIN              2

#define MAX_DIGITS_DEC       18
#define MAX_DIGITS_OCT       16
#define MAX_DIGITS_BIN       16
#define MAX_DIGITS_HEX        8
#define DISPLAY_LEN          18
#define HEX_OVERFLOW       2^32
#define DEC_OVERFLOW      10^18
#define KEY_BACKSPACE         8
#Define Key_Escape           27
#Define Key_Enter            13
#define ALLTRIM(x)        LTRIM(RTRIM(x))

**************************************
***
***   Calculator2Form
***
***   A simplilfied calculator for 
***   users who need basic operations
***   only.
***   Based on Calculator1Form (below)
**************************************

class Calculator1Form of FORM
   with (this)
      onGotFocus = class::FORMONGOTFOCUS
      canClose = class::FORMCANCLOSE
      onOpen = class::FORMOPEN
      //onClose = class::FORMONCLOSE
      scaleFontSize = 8
      scaleFontBold = false
      metric = 6	// Pixels
      colorNormal = "BTNTEXT/BTNFACE"
      height = 255 -55 -15
      //left = 658
      //top = 16
      width = 249 - 42 -12
      text = "Calculator"
      mdi = false
      sizeable = false
      escExit = false
      mousePointer = 1	// Arrow
      maximize = false
      AutoCenter = True
      smallTitle = true
   endwith

   this.DISPLAY = new TEXT(this)
   with (this.DISPLAY)
      height = 34
      //left = 36
      left = 1
      top = 6
      width = 211+36-42-1-12
      border = true
      colorNormal = "WindowText/Window"
      marginHorizontal = 1
      alignVertical = 1	// Middle
      alignHorizontal = 2	// Right
      text = "TEXT1"
      borderStyle = 4	// single
   endwith

   this.KEYHANDLER = new PAINTBOX(this)
   with (this.KEYHANDLER)
      onChar = class::KEYHANDLERONCHAR
      colorNormal = "BtnFace/BtnFace"
      height = 10
      left = 0
      top = 10
      width = 1
   endwith

   this.ABANDONBUTTON = new PUSHBUTTON(this)
   with (this.ABANDONBUTTON)
      onClick = {;Form.Close(False)}
      height = 20
      left = 42 -3
      top = 52
      width = 72 +6 -3
      text = "Cancel"
      speedBar = true
      upBitmap = "RESOURCE TS_ABANDON"
      fontSize = 8
      tabStop = false
      speedTip = "Close Form/Do Not Paste"
      group = true
   endwith

   this.BUTTONCLEAR = new PUSHBUTTON(this)
   with (this.BUTTONCLEAR)
      onClick = class::BUTTONCLEARONCLICK
      height = 20
      left = 211 - 42 -12
      top = 170 -15
      width = 36
      text = "CE\C"
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "WHITE/0XC0280"
   endwith

   this.B7 = new PUSHBUTTON(this)
   with (this.B7)
      onClick = class::DIGIT_ONCLICK
      height = 20
      left = 43  - 42
      top = 80
      width = 36
      text = "7"
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "B/BtnFace"
   endwith

   this.B8 = new PUSHBUTTON(this)
   with (this.B8)
      onClick = class::DIGIT_ONCLICK
      height = 20
      left = 85  - 42 -3
      top = 80
      width = 36
      text = "8"
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "B/BtnFace"
   endwith

   this.B9 = new PUSHBUTTON(this)
   with (this.B9)
      onClick = class::DIGIT_ONCLICK
      height = 20
      left = 127 - 42 -6 
      top = 80
      width = 36
      text = "9"
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "B/BtnFace"
   endwith

   this.B4 = new PUSHBUTTON(this)
   with (this.B4)
      onClick = class::DIGIT_ONCLICK
      height = 20
      left = 43  - 42
      top = 110-5
      width = 36
      text = "4"
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "B/BtnFace"
   endwith

   this.B5 = new PUSHBUTTON(this)
   with (this.B5)
      onClick = class::DIGIT_ONCLICK
      height = 20
      left = 85 - 42 -3
      top = 110 -5
      width = 36
      text = "5"
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "B/BtnFace"
   endwith

   this.B6 = new PUSHBUTTON(this)
   with (this.B6)
      onClick = class::DIGIT_ONCLICK
      height = 20
      left = 127  - 42 -6
      top = 110 -5
      width = 36
      text = "6"
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "B/BtnFace"
   endwith

   this.B1 = new PUSHBUTTON(this)
   with (this.B1)
      onClick = class::DIGIT_ONCLICK
      height = 20
      left = 43 - 42
      top = 140 -10
      width = 36
      text = "1"
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "B/BtnFace"
   endwith

   this.B2 = new PUSHBUTTON(this)
   with (this.B2)
      onClick = class::DIGIT_ONCLICK
      height = 20
      left = 85 - 42 -3
      top = 140 -10
      width = 36
      text = "2"
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "B/BtnFace"
   endwith

   this.B3 = new PUSHBUTTON(this)
   with (this.B3)
      onClick = class::DIGIT_ONCLICK
      height = 20
      left = 127 - 42 -6
      top = 140 -10
      width = 36
      text = "3"
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "B/BtnFace"
   endwith

   this.B0 = new PUSHBUTTON(this)
   with (this.B0)
      onClick = class::DIGIT_ONCLICK
      height = 20
      left = 43 - 42
      top = 170 -15
      width = 36
      text = "0"
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "B/BtnFace"
   endwith

   this.OPPLUSMINUS = new PUSHBUTTON(this)
   with (this.OPPLUSMINUS)
      onClick = class::OPPLUSMINUSONCLICK
      height = 20
      left = 85 - 42-3
      top = 170-15
      width = 36
      text = "+/-"
      speedBar = true
      fontSize = 8
      group = true
   endwith

   this.POINT = new PUSHBUTTON(this)
   with (this.POINT)
      onClick = class::POINTONCLICK
      height = 20
      left = 127 - 42 -6
      top = 170 -15
      width = 36
      text = ". "
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "B/BtnFace"
   endwith

   this.PASTEBUTTON = new PUSHBUTTON(this)
   with (this.PASTEBUTTON)
      onClick = class::PASTEBUTTON_ONCLICK
      height = 20
      left = 127 -9
      top = 51
      width = 72 +6 -3
      text = "Paste"
      speedBar = true
      upBitmap = "RESOURCE TS_paste"
      fontSize = 8
      tabStop = false
      speedTip = "Paste Result Into SpinBox"
      borderStyle = 1	// Raised
   endwith

   this.OPSQRT = new PUSHBUTTON(this)
   with (this.OPSQRT)
      onClick = class::OPSQRTONCLICK
      height = 20
      left = 211 - 42-12
      top = 80
      width = 36
      text = "sqrt"
      speedBar = true
      fontSize = 8
      group = true
   endwith

   this.OPPOWER = new PUSHBUTTON(this)
   with (this.OPPOWER)
      onClick = class::OPCLICK
      height = 20
      left = 211 - 42 -12
      top = 110 -5
      width = 36
      text = "^"
      speedBar = true
      fontSize = 8
      group = true
   endwith

   this.OPEQUAL = new PUSHBUTTON(this)
   with (this.OPEQUAL)
      onClick = class::OPCLICK
      height = 20
      left = 211 - 42-12
      top = 140-10
      width = 36
      text = "="
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "Darkgreen/BtnFace"
   endwith

   this.OPTIMES = new PUSHBUTTON(this)
   with (this.OPTIMES)
      onClick = class::OPCLICK
      height = 20
      left = 169 - 42-9
      top = 110 -5
      width = 36
      text = "X"
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "darkgreen/BtnFace"
   endwith

   this.OPDIV = new PUSHBUTTON(this)
   with (this.OPDIV)
      onClick = class::OPCLICK
      height = 20
      left = 169 - 42 -9
      top = 80
      width = 36
      text = "/"
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "darkgreen/BtnFace"
   endwith

   this.OPMINUS = new PUSHBUTTON(this)
   with (this.OPMINUS)
      onClick = class::OPCLICK
      height = 20
      left = 169  - 42-9
      top = 140-10
      width = 36
      text = "-"
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "darkgreen/BtnFace"
   endwith

   this.OPPLUS = new PUSHBUTTON(this)
   with (this.OPPLUS)
      onClick = class::OPCLICK
      height = 20
      left = 169 - 42-9
      top = 170  -15
      width = 36
      text = "+"
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "darkgreen/BtnFace"
   endwith

   function digit_onClick
      return ( class::enterDigit( this.text ) )

   function opPlusMinusOnClick
      //
      // Toggle sign of number in display.
      //

      local num

      num = class::NumVal( this.form.display.text )
      with ( this.form )
         display.text := class::charVal(num * -1)
         lastKeyOperator := false
      endwith
      return ( this.form.display.text )

   function opSqrtOnClick
      //
      // Replace current diplay with square root of display.
      //

      local num, sText
      num = class::NumVal( this.form.display.text )
      if ( num < 0 )
         this.form.display.text := REPLICATE( "*", MAX_DIGITS_DEC )
      else
         this.form.display.text := class::charVal( SQRT( num ) )
      endif
      this.form.lastKeyOperator := false
      return ( this.form.display.text )

   function enterDigit( sDigit )
      //
      // Process a digit.
      //
      local num

      Private ctrl_Name
      If sDigit $ '0123456789'
         ctrl_Name = 'Form.b' + sDigit
      Else If upper(sDigit) $ "ABCDEF"
         ctrl_Name = "Form.b_"+upper(sDigit)
      EndIf
      If Type('&ctrl_name') = 'O'
         &ctrl_Name..Value = True
         Sleep (.1)
         &ctrl_Name..Value = False
      EndIf

      if ( this.form.lastKeyOperator )
         with ( this.form )
            lastKeyOperator := false
            beforePoint     := true
            display.text    := space(MAX_DIGITS_DEC - 1) +;
                               class::displayValue( sDigit )
         endwith
      else
   
         do case
            case class::displayFull()
                 ?? chr(7)
            case this.form.beforePoint
                 // Only want rightmost DISPLAY_LEN characters
                 this.form.display.text := ;
                    right( class::displayValue( this.form.display.text ) + ;
                           class::displayValue( sDigit ), ;
                           DISPLAY_LEN )
            otherwise
                 this.form.display.text := ;
                    class::addAfterPoint( class::displayValue( sDigit ) )
         endcase

      endif


      return ( this.form.display.text )

   function opClick
      return ( class::enterOp( this ) )

   function pointOnClick
      //
      // Process decimal point.
      //

      Form.Point.Value = True
      Sleep(.1)
      Form.Point.Value = False

      if ( this.form.beforePoint )
         with ( this.form )
             beforePoint := false
             decPlaces    := 1
         endwith
         set decimals to 1
         if ( this.form.lastKeyOperator )
            with ( this.form )
               lastKeyOperator := false
               display.text := ;
                 space(MAX_DIGITS_DEC - 1) + SETTO("POINT")
            endwith
         else
            this.form.display.text := ;
               class::AddAfterPoint( SETTO("POINT") )
         endif
      Else
         ??chr(7)
      endif

      return ( this.form.display.text )

   function enterOp( thisOp )
      //
      // Process operator.
      //
      local lastOperation, lastOp, lastValue, tempOpList
      Private ctrl_Name
      ctrl_Name = ThisOp
      If Type('ctrl_Name') = 'O'
         ThisOp.Value = True
         Sleep (.1)
         ThisOp.Value = False
      EndIf

      if ( this.form.lastKeyOperator or ;
           this.form.operationStack.isEmpty() )
         this.form.operationStack.pop()
         this.form.lastValue := class::numVal( this.form.display.text )
      else
         // Process all previous operations with >= precedence
         do while ( this.form.operationStack.prevPrecedenceGreaterOrEqual( ;
                    thisOp.precedence ) )
            lastOperation = this.form.operationStack.pop()

            if ( lastOperation.getPrecedence() > 0 ) // Don't process "="
               lastOp    = lastOperation.getOp()
               lastValue = lastOperation.getValue()
               set decimals to this.form.mostDecPlaces
               this.form.lastValue := ;
                  lastOp( lastValue, class::numVal( this.form.display.text ) )
               this.form.display.text := class::charVal( this.form.lastValue )
               this.form.decPlaces    := 1
               set decimals to 1
            endif
         enddo
      endif
      with ( this.form )
         beforePoint     := true
         lastKeyOperator := true
      endwith

      thisOp.opValue = class::numVal( this.form.display.text )
      this.form.operationStack.push( ( thisOp ) )

      return ( thisOp.opValue )

   function displayFull
      //
      // Check if display already has MAX_DIGITS_DEC digits in it
      //

      local bFull, maxValueLen
      bFull       = false
      maxValueLen = MAX_DIGITS_DEC
      do case
         case ( this.form.numberSys == BASE_HEX )
              maxValueLen := MAX_DIGITS_HEX
         case ( this.form.numberSys == BASE_OCT )
              maxValueLen := MAX_DIGITS_OCT
         otherwise
              maxValueLen := MAX_DIGITS_DEC
      endcase

      // Check if leftmost digit in current display = " "
      bFull := ( not empty( left( right( this.form.display.text, ;
                                maxValueLen ), 1) ) )

      return ( bFull )

   function addAfterPoint( char )
      //
      // Add fractional digits after decimal point.
      //
      this.form.decPlaces := this.form.decPlaces + 1
      set decimals to this.form.decPlaces
      this.form.mostDecPlaces := max( this.form.decPlaces, ;
                                      this.form.mostDecPlaces )

      return ( class::DisplayValue( this.form.display.text ) + char )

   function buttonClearOnClick
      //
      // Set custom form properties to default values
      // and clear display..
      //
      Form.ButtonClear.Value = True
      Sleep(.1)
      Form.ButtonClear.Value = False

      this.form.operationStack.Initialize() // Empty operation stack         
      set decimals to 1                     // Initial decimal places
      with ( this.form )
           lastValue       := 0        // Clear value
           lastKeyOperator := null     // Clear operators
           decPlaces       := 1  
           mostDecPlaces   := 1
           beforePoint     := true     // Start with whole number entry
           display.text    := SPACE( MAX_DIGITS_DEC - 1 ) + "0"
      endwith   
      return ( this.form.display.text )

   function displayValue( value )
      //
      //  Display value without the pick character.
      //
      local num, pickLoc
      num = value
      pickLoc = at("&",num)

      do case
         case ( pickLoc <> 0 )
              num := stuff(num,pickLoc,1,"")
         case ( right(num,2) = " 0" )
              num := space(MAX_DIGITS_DEC)
         case ( left(num,1) = SPACE(1) )
              num := substr(num,2)
       endcase

      return ( num )

   Function FormCanClose
      If Type('Form.Save_Decimals') <> 'U'
         Set Decimals to  Form.Save_Decimals
      EndIf
      If Type ('Form.Save_CuaEnter') <> 'U'
      	If Form.Save_CuaEnter = 'ON'
		      Set CUAEnter On
         Else
         	Set CUAEnter Off
         EndIf
      EndIf
      //For my in-house use!
      If type('_app.ini') <> 'U'
      	Set Procedure to :dulfp:Center32.prg additive
         _app.ini.SetValue(id(),'Calculator',SetFrame(Form.Hwnd,0,0))
         _app.ini.Flush()
         Close Procedure :dulfp:Center32.prg
      EndIf
      Return True

   Function formOnClose
      //
      // Clean up
      //
      return //( true )

   function formOnGotFocus
      //
      // Make sure decimal places is correct every time 
      // this form gets focus.
      //
      If Type('Form.Save_Decimals') = 'U'
         Form.Save_Decimals = Setto('Decimals')
      EndIf

      If Type ('Form.Save_CuaEnter') = 'U'
	      Form.Save_CuaEnter =  Set('CuaEnter')
      EndIf
      Set CUAEnter On


      If Type('This.DecPlaces') = 'U'
         //Form onopen not yet processed
         Set Decimals to 1
      Else
         set decimals to this.decPlaces
      EndIf //Type('This.DecPlaces') = 'U'
      Form.Keyhandler.SetFocus()
      return //( SET("DECIMALS") )

   function keyHandlerOnChar(nChar, nRepCnt, nFlags)
      local sChar, vChar
      sChar = UPPER( CHR( nChar ) )
      vChar = VAL( sChar )
      if ( vChar > 0 AND vChar < this.form.numberSys )  OR ;
         ( sChar $ 'ABCDEF' AND form.numberSys = BASE_HEX)
         class::enterDigit( sChar )
      else
         do case
            case ( sChar == "0" )
                 class::enterDigit( sChar )
            case ( sChar == "/" )
                 class::enterOp( this.form.opDiv )
            case ( ( sChar == "*" ) or ;
                   ( sChar == "X" ) )
                 class::enterOp( this.form.opTimes )
            case ( sChar == "-" )
                 class::enterOp( this.form.opMinus )
            case ( sChar == "+" )
                 class::enterOp( this.form.opPlus )
            case ( sChar == "^" )
                 class::enterOp( this.form.opPower )
            case ( ( sChar == "=" ) or ;
                   ( nChar == key_enter  ) )
                 class::enterOp( this.form.opEqual )
            case ( ( sChar == SETTO("POINT") ) and ;
                   ( this.form.point.enabled ) )
                 class::pointOnClick()
            case ( nChar == Key_Escape )
                 class::BUTTONCLEARONCLICK()
            case ( nChar == KEY_BACKSPACE )
                 class::removeLastDigit()
         endcase
      endif
      return // (nChar)

   function removeLastDigit
      //
      // Remove last digit for backspace key
      //
      if ( not this.form.lastKeyOperator )
 
         if ( not ( class::numVal( this.form.display.text ) == 0 ) )
            this.form.display.text := ;
               this.form.display.text.substring( 0, this.form.display.text.length - 1 )
         endif

         if ( class::numVal( this.form.display.text ) == 0 )
            this.form.display.text := SPACE( MAX_DIGITS_DEC - 1 ) + "0"
         endif

      endif

      return true


   function formOpen

      If Type ('Form.Save_CuaEnter') = 'U'
	      Form.Save_CuaEnter =  Set('CuaEnter')
      EndIf
      Set CUAEnter On

      If Type('Form.Save_Decimals') = 'U'
         Form.Save_Decimals = Setto('Decimals')
      EndIf
      set decimals to 1 

      this.point.text := SETTO("POINT") // Match current country  "," or "."

      this.opPlus.doIt        = { |a,b| ; return ( a + b ) }
      this.opPlus.precedence  = 1

      this.opMinus.doIt       = { |a,b| ; return ( a - b ) }
      this.opMinus.precedence = 1

      this.opTimes.doIt       = { |a,b| ; return ( a * b ) }
      this.opTimes.precedence = 2

      this.opDiv.doIt         = class::opDivideIt
      this.opDiv.precedence   = 2

      this.opPower.doIt       = { |a,b| ; return ( a ^ b ) }
      this.opPower.precedence = 3

      this.opEqual.doIt       = { |a,b| ; return null }
      this.opEqual.precedence = 0

      //
      // Create custom form properties
      // 
      // Stack for storing operation states
      this.operationStack = new OperationStackClass()
      this.operationStack.Initialize() // Empty operation stack         
 
      this.numberSys = BASE_DEC 
      this.decPlaces      = 1    // Start with minimum decimal places
      this.mostDecPlaces  = 1    // Most decimal places in an operand

      this.beforePoint     = true // Add numbers before or after decimal pt.
      this.lastKeyOperator = null
      this.lastValue       = 0        
      this.KEYHANDLER.setFocus()

      return //( CALCULATORFORM::OPEN() )

   Function Pastebutton_Onclick
   	If Form.NumberSys <> Base_Dec
      	If Msgbox('Convert result to decimal?','Please Verify', 32 + 4) = 6
         	//yes
            class::changeBase( BASE_DEC )
         Else
         	MsgBox('Cannot paste non-decimal number to spinbox.',;
                   'Abandoning changes', 64)
            form.Close(False)
            Return
         EndIf //Msgbox('Convert result to decimal?','Please Verify', 32 + 4) = 6
      EndIf //Form.NumberSys <> Base_Dec
      Form.Close(Val(LTRIM( Form.display.text )))

   function opDivideIt( a, b )
      local nResult
      nResult = 0
      if ( b == 0 )
         MSGBOX("Cannot divide by zero.", "Alert",48)
      else
         nResult := ( a / b )
      endif
      return ( nResult )

   function charVal( num )
      //
      // Convert decimal number to string using 
      // current base: hex, decimal, octal, binary.
      //
      local string, fractionVal
      local nPlace
      local nPValue
      string = ""

      do case
         case ( this.form.numberSys == BASE_HEX )

              if ( ABS(num) >= HEX_OVERFLOW )
                 string := replicate("*", MAX_DIGITS_HEX)
              else
                 string := itoh(num)
                 string := space(MAX_DIGITS_DEC - len(string)) + string
              endif

         case ( this.form.numberSys == BASE_OCT )
              string := class::toBaseString( num, 16, BASE_OCT )

         case ( this.form.numberSys == BASE_BIN )
              string := class::toBaseString( num, 16, BASE_BIN )

         case ( this.form.numberSys == BASE_DEC )
              if ( ABS(num) >= DEC_OVERFLOW )
                 string := replicate("*", MAX_DIGITS_DEC)
              else
                 string := class::decimalShowString( num )
              endif

      endcase
      return ( string )

   function decimalShowString( n )
      local nDec, sInt, sMask, sVal     
      nDec = SET("DECIMALS")
      sInt  = ""
      sMask = ""
      sVal  = ""
      SET DECIMALS TO 18

      if ( n < 0 ) and ( n > -1 )
         sInt := "-0"
      else
         sInt := int( n )+"".rightTrim()
      endif

      sMask := replicate("9", sInt.length ) + "." + ;
                 replicate("9", ( 14 - sInt.length ) )
      sVal := TRANSFORM(n,sMask)

      do while ( SETTO("POINT") $ sVal ) and ( sVal.right(1) == "0" )
         sVal := sVal.substring( 0, sVal.length - 1 )
      enddo

      if sVal.right(1) == SETTO("POINT")
         sVal := sVal.substring( 0, sVal.length - 1 )
      endif

      SET DECIMALS TO ( nDec )
      return ( sVal )

   function toBaseString( nDecValue, nMaxPlace, nBase )
      local nPlace
      local npValue
      local nAdd
      local sValue
      local bStart
      local bCarry

      nPlace  = nMaxPlace
      npValue = 0
      sValue  = ""
      bStart  = false
      if ( ABS( nDecValue ) >= nBase ^ nPlace )
         sValue = REPLICATE("*", nMaxPlace )
      else                
         do while ( nPlace >= 0 )
            if ( nDecValue == 0 )
               sValue := sValue + "0"
            else
               npValue := ( nBase ^ nPlace )
               if ( ( npValue <= nDecValue ) or ;
                    ( npValue == 1 ) )
                  bStart := true
                  if ( nDecValue / npValue ) == nBase
                     sValue := sValue + "10"
                  else
                     sValue := sValue + ;
                              ALLTRIM( STR( INT( nDecValue / npValue )  ) )
                  endif
                  nDecValue := MOD( nDecValue, npValue )
               else
                  if ( bStart )
                     sValue := sValue + "0"
                  endif
               endif
            endif
            nPlace := nPlace - 1
         enddo
         if ( VAL( sValue ) == 0 )
            sValue := "0"
         endif
         sValue := SPACE( MAX_DIGITS_DEC - LEN( sValue ) ) + sValue
      endif
      return ( sValue )

   function numVal( string )
      //
      // Return decimal value of display value using
      // current base to translate.
      //
      local h, num, nPoint
      local sValue
      local nDigit, nPlace
      private macro

      sValue  = ALLTRIM( string )
      h       = ""
      macro   = ""
      num     = 0
      nDigit  = 0
      nPlace  = 0
      nPoint  = 0

      do case

         case ( this.form.numberSys == BASE_HEX )

              h   := htoi(string)
              num := iif(h >= HEX_OVERFLOW/2, bitxor(h, HEX_OVERFLOW), h)

         case ( this.form.numberSys == BASE_OCT )

              if ( sValue == null )
                 num := 0
              else
                 if ( SUBSTR( sValue, 1, 1 ) == "*" )
                    num := ( 1 / 0 )
                 else
                    macro := "0" + sValue
                    num := &macro.
                 endif
              endif

         case ( this.form.numberSys == BASE_BIN )

              do while ( LEN( sValue ) > 0 )
                 nDigit := IIF( RIGHT( sValue, 1 ) == "1", 1, 0 )
                 sValue := SUBSTR( sValue, 1, ( LEN( sValue ) - 1 ) )
                 num    := num + ( nDigit * (2 ^ nPlace) )
                 nPlace := nPlace + 1
              enddo

         otherwise            
              // decimal
              nPoint := AT( SETTO("POINT"), sValue )

              if ( nPoint <> 0  )               
                 num := VAL( STUFF( sValue, nPoint, 1, SETTO("POINT") ) )
              else
                 num := VAL( sValue )
              endif

      endcase
      return ( num )

endclass


**************************************
***
***   Calculator2Form
***
**************************************
//------------------------------------------------------------------------
//
//  This file contains a calculator form.  This calculator
//  contains pushbuttons for numeric input and operations.  It 
//  also contains 2 entryfields -- the main display, showing the 
//  current calculation, a hidden entry field for storing values 
//  to memory. Keyboard input is handled through a paintbox control
//  that is not visible to the user.
//
//  dBASE Samples Group
//
//  $Revision:   1.18  $
//
//  Copyright (c) 1997-1998, Borland International, Inc. 
//  Copyright (c) 2000, dBase, Inc. 
//  All rights reserved.
//
//  Revisions 3/13/2001 By Paul Van House noted above
//
//------------------------------------------------------------------------
//
/*

    Revision: 5/18/01 Removed code that caused calculator to maximize
                      when MDI windows in _app.framewin were maximized
    Revision: 3/14/01 Fixed a couple of typos when setting properties,
                      Revised positioning routine to account for title
                   and menu bars on the calculator form.
                      Added a Save CuaEnter field so Set CUAEnter can be
                   on for the calculator form to allow use of enter key
                   as an equivialent for the = key.

    Platform: Visual dBASE 32 bit
    Written:  03/13/2001
    By:       Paul Van House (pvanhouse@fvc.cc -- http://pvhbinx.hypermart.net

    Derived from and replaces Calculator.WFM -- from db2K Samples group 

    You may also call the calculator by itself without it being attached to a
    spinbox by doing the following:
               Set Procedure to Calculatorspin.cc Additive
               x = new CalculatorForm()
               x.ReadModal()
    When used in this fashion, the menu option to "paste to spinbox" is disabled, 
    and the button used to "paste to the spinbox" becomes a "copy to clipboard"
    button. 

    CHANGES MADE TO CALCULATOR.WFM BY PAUL VAN HOUSE 3/13/01:
    * Modified to work as a custom class
    * Changed form layout to more closely match appearance of the "default" 
      windows calculator.
    * Added visual effects to highlight buttons when keyboard is used
      to generate input: All Number buttons, A-F when in hex mode, decimal
      point, All operator buttons that are on keyboard (+-=^/*x)
    * Added keyboard "Escape" to key handler as a "clear" button as in
      "default" windows calculator.
    * Changed form.metric from 0 (chars) to 6 (pixels) for ease of positioning
    * Added "abandon" button to close form.
    * Added new form field "lLink" as a logical field to indicate if the
      form was called from the custom control (lLink = True).  If form was just 
      called without a spinbox reference, lLink defaults to False.
    * Added "Paste" button to return current shown value to calling routine.
      typically used to populate spinbox with current shown value. (lLink = True)
      If NOT called as a custom control (lLink = False) , this button will copy
      the current shown value into the windows clipboard.
    * Added error checking to OnGotFocus method to be sure form.DecPlaces
      has already been defined.  When run as a "modal" form, the OnGotFocus
      apparently fired before the Form's OnOpen method!
    * Added procedure to preserve the Set Decimals setting.
      If called from the spinbox (lLink = True) and paste is attempted with
      a non base-10 number, the user is given a chance to convert the number
      to base 10 (decimal).

*/
class Calculator2Form of FORM
   with (this)
      onGotFocus = class::FORMONGOTFOCUS
      canClose = class::FORMCANCLOSE
      onOpen = class::FORMOPEN
      onClose = class::FORMONCLOSE
      scaleFontSize = 8
      scaleFontBold = false
      metric = 6	// Pixels
      colorNormal = "BTNTEXT/BTNFACE"
      height = 200
      left = 658
      top = 16
      width = 248
      text = "Calculator"
      mdi = false
      sizeable = false
      smallTitle = true
      escExit = false
      mousePointer = 1	// Arrow
      maximize = false
      //icon = "FILE Calculator.ico"
      AutoCenter = True
   endwith

   this.TEXTMINDICATOR = new TEXT(this)
   with (this.TEXTMINDICATOR)
      height = 34
      left = 7
      top = 6
      width = 24
      colorNormal = "BtnText"
      alignVertical = 1	// Middle
      text = " "
   endwith

   this.DISPLAY = new TEXT(this)
   with (this.DISPLAY)
      height = 34
      left = 36
      top = 6
      width = 210
      border = true
      colorNormal = "WindowText/Window"
      marginHorizontal = 1
      alignVertical = 1	// Middle
      alignHorizontal = 2	// Right
      text = "TEXT1"
      borderStyle = 5	// Double
   endwith

   this.KEYHANDLER = new PAINTBOX(this)
   with (this.KEYHANDLER)
      onChar = class::KEYHANDLERONCHAR
      colorNormal = "BtnFace/BtnFace"
      height = 10
      left = 0
      top = 10
      width = 1
   endwith

   this.RECTANGLE1 = new RECTANGLE(this)
   with (this.RECTANGLE1)
      left = 0
      top = 48
      width = 162
      height = 25
      text = ""
      colorNormal = "WindowText/BtnFace"
      fontSize = 8
   endwith

   this.RADIOHEX = new RADIOBUTTON(this)
   with (this.RADIOHEX)
      onGotFocus = {; this.form.KEYHANDLER.setFocus()}
      onChange = class::RADIOHEXONCHANGE
      height = 16
      left = 4
      top = 54
      width = 36
      text = "Hex"
      colorNormal = "WindowText/BtnFace"
      fontSize = 8
      tabStop = false
      group = true
   endwith

   this.RADIODEC = new RADIOBUTTON(this)
   with (this.RADIODEC)
      onGotFocus = {; this.form.KEYHANDLER.setFocus()}
      onChange = class::RADIODECONCHANGE
      height = 16
      left = 44
      top = 54
      width = 40
      text = "Dec"
      colorNormal = "WindowText/BtnFace"
      fontSize = 8
      tabStop = false
      value = true
   endwith

   this.RADIOOCT = new RADIOBUTTON(this)
   with (this.RADIOOCT)
      onGotFocus = {; this.form.KEYHANDLER.setFocus()}
      onChange = class::RADIOOCTONCHANGE
      height = 16
      left = 84
      top = 54
      width = 39
      text = "Oct"
      colorNormal = "WindowText/BtnFace"
      fontSize = 8
      tabStop = false
   endwith

   this.RADIOBIN = new RADIOBUTTON(this)
   with (this.RADIOBIN)
      onGotFocus = {; this.form.KEYHANDLER.setFocus()}
      onChange = class::RADIOBINONCHANGE
      height = 16
      left = 126
      top = 54
      width = 32
      text = "Bin"
      colorNormal = "WindowText/BtnFace"
      fontSize = 8
      tabStop = false
   endwith

   this.ABANDONBUTTON = new PUSHBUTTON(this)
   with (this.ABANDONBUTTON)
      onClick = {;Form.Close(False)}
      height = 20
      left = 168
      top = 51
      width = 36
      text = ""
      speedBar = true
      upBitmap = "RESOURCE TS_ABANDON"
      fontSize = 8
      tabStop = false
      speedTip = "Close Form/Do Not Paste"
      group = true
   endwith

   this.BUTTONCLEAR = new PUSHBUTTON(this)
   with (this.BUTTONCLEAR)
      onClick = class::BUTTONCLEARONCLICK
      height = 20
      left = 210
      top = 170
      width = 36
      text = "CE\C"
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "WHITE/0XC0280"
   endwith

   this.B7 = new PUSHBUTTON(this)
   with (this.B7)
      onClick = class::DIGIT_ONCLICK
      height = 20
      left = 42
      top = 80
      width = 36
      text = "7"
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "B/BtnFace"
   endwith

   this.B8 = new PUSHBUTTON(this)
   with (this.B8)
      onClick = class::DIGIT_ONCLICK
      height = 20
      left = 84
      top = 80
      width = 36
      text = "8"
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "B/BtnFace"
   endwith

   this.B9 = new PUSHBUTTON(this)
   with (this.B9)
      onClick = class::DIGIT_ONCLICK
      height = 20
      left = 126
      top = 80
      width = 36
      text = "9"
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "B/BtnFace"
   endwith

   this.B4 = new PUSHBUTTON(this)
   with (this.B4)
      onClick = class::DIGIT_ONCLICK
      height = 20
      left = 42
      top = 110
      width = 36
      text = "4"
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "B/BtnFace"
   endwith

   this.B5 = new PUSHBUTTON(this)
   with (this.B5)
      onClick = class::DIGIT_ONCLICK
      height = 20
      left = 84
      top = 110
      width = 36
      text = "5"
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "B/BtnFace"
   endwith

   this.B6 = new PUSHBUTTON(this)
   with (this.B6)
      onClick = class::DIGIT_ONCLICK
      height = 20
      left = 126
      top = 110
      width = 36
      text = "6"
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "B/BtnFace"
   endwith

   this.B1 = new PUSHBUTTON(this)
   with (this.B1)
      onClick = class::DIGIT_ONCLICK
      height = 20
      left = 42
      top = 140
      width = 36
      text = "1"
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "B/BtnFace"
   endwith

   this.B2 = new PUSHBUTTON(this)
   with (this.B2)
      onClick = class::DIGIT_ONCLICK
      height = 20
      left = 84
      top = 140
      width = 36
      text = "2"
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "B/BtnFace"
   endwith

   this.B3 = new PUSHBUTTON(this)
   with (this.B3)
      onClick = class::DIGIT_ONCLICK
      height = 20
      left = 126
      top = 140
      width = 36
      text = "3"
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "B/BtnFace"
   endwith

   this.B0 = new PUSHBUTTON(this)
   with (this.B0)
      onClick = class::DIGIT_ONCLICK
      height = 20
      left = 42
      top = 170
      width = 36
      text = "0"
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "B/BtnFace"
   endwith

   this.OPPLUSMINUS = new PUSHBUTTON(this)
   with (this.OPPLUSMINUS)
      onClick = class::OPPLUSMINUSONCLICK
      height = 20
      left = 84
      top = 170
      width = 36
      text = "+/-"
      speedBar = true
      fontSize = 8
      group = true
   endwith

   this.POINT = new PUSHBUTTON(this)
   with (this.POINT)
      onClick = class::POINTONCLICK
      height = 20
      left = 126
      top = 170
      width = 36
      text = ". "
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "B/BtnFace"
   endwith

   this.B_A = new PUSHBUTTON(this)
   with (this.B_A)
      onClick = class::DIGIT_ONCLICK
      height = 20
      left = 0
      top = 200
      width = 36
      text = "A"
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "Blue/BtnFace"
   endwith

   this.B_B = new PUSHBUTTON(this)
   with (this.B_B)
      onClick = class::DIGIT_ONCLICK
      height = 20
      left = 42
      top = 200
      width = 36
      text = "B"
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "Blue/BtnFace"
   endwith

   this.B_C = new PUSHBUTTON(this)
   with (this.B_C)
      onClick = class::DIGIT_ONCLICK
      height = 20
      left = 84
      top = 200
      width = 36
      text = "C"
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "Blue/BtnFace"
   endwith

   this.B_D = new PUSHBUTTON(this)
   with (this.B_D)
      onClick = class::DIGIT_ONCLICK
      height = 20
      left = 126
      top = 200
      width = 36
      text = "D"
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "Blue/BtnFace"
   endwith

   this.B_E = new PUSHBUTTON(this)
   with (this.B_E)
      onClick = class::DIGIT_ONCLICK
      height = 20
      left = 168
      top = 200
      width = 36
      text = "E"
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "Blue/BtnFace"
   endwith

   this.B_F = new PUSHBUTTON(this)
   with (this.B_F)
      onClick = class::DIGIT_ONCLICK
      height = 20
      left = 210
      top = 200
      width = 36
      text = "F"
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "Blue/BtnFace"
   endwith

   this.PASTEBUTTON = new PUSHBUTTON(this)
   with (this.PASTEBUTTON)
      onClick = class::PASTEBUTTON_ONCLICK
      height = 20
      left = 210
      top = 51
      width = 36
      text = ""
      speedBar = true
      upBitmap = "RESOURCE TS_paste"
      fontSize = 8
      tabStop = false
      speedTip = "Paste Result Into SpinBox"
      borderStyle = 1	// Raised
   endwith

   this.OPSQRT = new PUSHBUTTON(this)
   with (this.OPSQRT)
      onClick = class::OPSQRTONCLICK
      height = 20
      left = 210
      top = 80
      width = 36
      text = "sqrt"
      speedBar = true
      fontSize = 8
      group = true
   endwith

   this.OPPOWER = new PUSHBUTTON(this)
   with (this.OPPOWER)
      onClick = class::OPCLICK
      height = 20
      left = 210
      top = 110
      width = 36
      text = "^"
      speedBar = true
      fontSize = 8
      group = true
   endwith

   this.OPEQUAL = new PUSHBUTTON(this)
   with (this.OPEQUAL)
      onClick = class::OPCLICK
      height = 20
      left = 210
      top = 140
      width = 36
      text = "="
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "Darkgreen/BtnFace"
   endwith

   this.MC = new PUSHBUTTON(this)
   with (this.MC)
      onClick = class::MCONCLICK
      height = 20
      left = 0
      top = 80
      width = 36
      text = "MC"
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "WHITE/0X804000"
   endwith

   this.MR = new PUSHBUTTON(this)
   with (this.MR)
      onClick = class::MRONCLICK
      height = 20
      left = 0
      top = 110
      width = 36
      text = "MR"
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "WHITE/0X804000"
   endwith

   this.MADD = new PUSHBUTTON(this)
   with (this.MADD)
      onClick = class::MEMONCLICK
      height = 20
      left = 0
      top = 140
      width = 36
      text = "M+"
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "WHITE/0X804000"
   endwith

   this.MSUB = new PUSHBUTTON(this)
   with (this.MSUB)
      onClick = class::MEMONCLICK
      height = 20
      left = 0
      top = 170
      width = 36
      text = "M-"
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "WHITE/0X804000"
   endwith

   this.MEMORY = new ENTRYFIELD(this)
   with (this.MEMORY)
      enabled = false
      height = 20
      left = 0
      top = 230
      width = 246
      function = "J"
      colorNormal = "BtnText/BtnFace"
      fontSize = 8
      value = "                  "
      validRequired = true
      borderStyle = 7	// Client
   endwith

   this.OPTIMES = new PUSHBUTTON(this)
   with (this.OPTIMES)
      onClick = class::OPCLICK
      height = 20
      left = 168
      top = 110
      width = 36
      text = "X"
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "darkgreen/BtnFace"
   endwith

   this.OPDIV = new PUSHBUTTON(this)
   with (this.OPDIV)
      onClick = class::OPCLICK
      height = 20
      left = 168
      top = 80
      width = 36
      text = "/"
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "darkgreen/BtnFace"
   endwith

   this.OPMINUS = new PUSHBUTTON(this)
   with (this.OPMINUS)
      onClick = class::OPCLICK
      height = 20
      left = 168
      top = 140
      width = 36
      text = "-"
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "darkgreen/BtnFace"
   endwith

   this.OPPLUS = new PUSHBUTTON(this)
   with (this.OPPLUS)
      onClick = class::OPCLICK
      height = 20
      left = 168
      top = 170
      width = 36
      text = "+"
      speedBar = true
      fontSize = 8
      group = true
      colorNormal = "darkgreen/BtnFace"
   endwith


   function changeBase( newBase )
      //
      // Change base number system
      //
      local value, memValue, dispFraction, memFraction

      // Get values in display and memory before changing 
      // hex indicator
      value    = CLASS::numVal( this.form.display.text )
      memValue = CLASS::numVal( this.form.memory.value )
      this.form.beforePoint := true
      SET DECIMALS TO this.form.decPlaces

      dispFraction = val(substr( this.form.display.text,;
             at( SETTO("POINT"), this.form.display.text)))
      memFraction  = val(substr(form.memory.value,;
             at( SETTO("POINT"), this.form.memory.value)))

      this.form.numberSys := newBase 

      if ( abs(value) >= HEX_OVERFLOW )
         // Temporary
         this.form.display.text := space(DISPLAY_LEN - MAX_DIGITS_HEX) +;
                             replicate("*", MAX_DIGITS_HEX)
      else
         this.form.display.text := CLASS::charVal( value + dispFraction )
      endif

      if ( abs(memValue) >= HEX_OVERFLOW )
         this.form.memory.value  := space(DISPLAY_LEN - MAX_DIGITS_HEX) +;
                             replicate("*", MAX_DIGITS_HEX)
      else
         this.form.memory.value := CLASS::charVal( memValue + memFraction )
      endif

   return ( this.form.numberSys )

   function digit_onClick
   return ( class::enterDigit( this.text ) )

   function opPlusMinusOnClick
      //
      // Toggle sign of number in display.
      //

      local num

      num = class::NumVal( this.form.display.text )
      with ( this.form )
         display.text := class::charVal(num * -1)
         lastKeyOperator := false
      endwith
   return ( this.form.display.text )

   function opSqrtOnClick
      //
      // Replace current diplay with square root of display.
      //

      local num, sText
      num = class::NumVal( this.form.display.text )
      if ( num < 0 )
         this.form.display.text := REPLICATE( "*", MAX_DIGITS_DEC )
      else
         this.form.display.text := class::charVal( SQRT( num ) )
      endif
      this.form.lastKeyOperator := false
   return ( this.form.display.text )

   function enterDigit( sDigit )
      //
      // Process a digit.
      //
      local num

      Private ctrl_Name
      If sDigit $ '0123456789'
         ctrl_Name = 'Form.b' + sDigit
      Else If upper(sDigit) $ "ABCDEF"
         ctrl_Name = "Form.b_"+upper(sDigit)
      EndIf
      If Type('&ctrl_name') = 'O'
         &ctrl_Name..Value = True
         Sleep (.1)
         &ctrl_Name..Value = False
      EndIf

      if ( this.form.lastKeyOperator )
         with ( this.form )
            lastKeyOperator := false
            beforePoint     := true
            display.text    := space(MAX_DIGITS_DEC - 1) +;
                               class::displayValue( sDigit )
         endwith
      else
   
         do case
            case class::displayFull()
                 ?? chr(7)
            case this.form.beforePoint
                 // Only want rightmost DISPLAY_LEN characters
                 this.form.display.text := ;
                    right( class::displayValue( this.form.display.text ) + ;
                           class::displayValue( sDigit ), ;
                           DISPLAY_LEN )
            otherwise
                 this.form.display.text := ;
                    class::addAfterPoint( class::displayValue( sDigit ) )
         endcase

      endif


   return ( this.form.display.text )

   function opClick
   return ( class::enterOp( this ) )

   function pointOnClick
      //
      // Process decimal point.
      //

      Form.Point.Value = True
      Sleep(.1)
      Form.Point.Value = False

      if ( this.form.beforePoint )
         with ( this.form )
             beforePoint := false
             decPlaces    := 1
         endwith
         set decimals to 1
         if ( this.form.lastKeyOperator )
            with ( this.form )
               lastKeyOperator := false
               display.text := ;
                 space(MAX_DIGITS_DEC - 1) + SETTO("POINT")
            endwith
         else
            this.form.display.text := ;
               class::AddAfterPoint( SETTO("POINT") )
         endif
      Else
         ??chr(7)
      endif

   return ( this.form.display.text )

   function enterOp( thisOp )
      //
      // Process operator.
      //
      local lastOperation, lastOp, lastValue, tempOpList
      Private ctrl_Name
      ctrl_Name = ThisOp
      If Type('ctrl_Name') = 'O'
         ThisOp.Value = True
         Sleep (.1)
         ThisOp.Value = False
      EndIf

      if ( this.form.lastKeyOperator or ;
           this.form.operationStack.isEmpty() )
         this.form.operationStack.pop()
         this.form.lastValue := class::numVal( this.form.display.text )
      else
         // Process all previous operations with >= precedence
         do while ( this.form.operationStack.prevPrecedenceGreaterOrEqual( ;
                    thisOp.precedence ) )
            lastOperation = this.form.operationStack.pop()

            if ( lastOperation.getPrecedence() > 0 ) // Don't process "="
               lastOp    = lastOperation.getOp()
               lastValue = lastOperation.getValue()
               set decimals to this.form.mostDecPlaces
               this.form.lastValue := ;
                  lastOp( lastValue, class::numVal( this.form.display.text ) )
               this.form.display.text := class::charVal( this.form.lastValue )
               this.form.decPlaces    := 1
               set decimals to 1
            endif
         enddo
      endif
      with ( this.form )
         beforePoint     := true
         lastKeyOperator := true
      endwith

      thisOp.opValue = class::numVal( this.form.display.text )
      this.form.operationStack.push( ( thisOp ) )

   return ( thisOp.opValue )

   function memOnClick
      //
      // Process a memory action button.
      //

      local result

      result = this.doIt( class::numVal( this.form.memory.value ),;
                          class::numVal( this.form.display.text ))
      with ( this.form )
         memory.value := class::CharVal(result)
         textMindicator.text := "M"
      endwith
   return ( result )

   function radioBinOnChange       

      with ( this.form )
         b2.enabled := ( not this.value )
         b3.enabled := ( not this.value )
         b4.enabled := ( not this.value )
         b5.enabled := ( not this.value )
         b6.enabled := ( not this.value )
         b7.enabled := ( not this.value )
         b8.enabled := ( not this.value )
         b9.enabled := ( not this.value )
      endwith

      if ( this.value )
         class::changeBase( BASE_BIN )
      endif

   return ( this.value )

   function radioDecOnChange
      // Enable keys applicable to decimal calculations
      with ( this.form )
         opPlusMinus.enabled := ( this.value )
         point.enabled       := ( this.value )  
      endwith

      if ( this.value )
         class::changeBase( BASE_DEC )
      endif

   return ( this.value )

   function radioHexOnChange      

      if ( this.value )
         this.form.height := 230
         class::changeBase( BASE_HEX )    
      else
         this.form.height := 200
      endif  
   return ( this.value )

   function radioOctOnChange

      with ( this.form )
         b8.enabled := ( not this.value )
         b9.enabled := ( not this.value )
      endwith

      if ( this.value )
         class::changeBase( BASE_OCT )
      endif

   return ( this.value )

   function displayFull
      //
      // Check if display already has MAX_DIGITS_DEC digits in it
      //

      local bFull, maxValueLen
      bFull       = false
      maxValueLen = MAX_DIGITS_DEC
      do case
         case ( this.form.numberSys == BASE_HEX )
              maxValueLen := MAX_DIGITS_HEX
         case ( this.form.numberSys == BASE_OCT )
              maxValueLen := MAX_DIGITS_OCT
         otherwise
              maxValueLen := MAX_DIGITS_DEC
      endcase

      // Check if leftmost digit in current display = " "
      bFull := ( not empty( left( right( this.form.display.text, ;
                                maxValueLen ), 1) ) )

   return ( bFull )

   function addAfterPoint( char )
      //
      // Add fractional digits after decimal point.
      //
      this.form.decPlaces := this.form.decPlaces + 1
      set decimals to this.form.decPlaces
      this.form.mostDecPlaces := max( this.form.decPlaces, ;
                                      this.form.mostDecPlaces )

   return ( class::DisplayValue( this.form.display.text ) + char )

   function buttonClearOnClick
      //
      // Set custom form properties to default values
      // and clear display..
      //
      Form.ButtonClear.Value = True
      Sleep(.1)
      Form.ButtonClear.Value = False

      this.form.operationStack.Initialize() // Empty operation stack         
      set decimals to 1                     // Initial decimal places
      with ( this.form )
           lastValue       := 0        // Clear value
           lastKeyOperator := null     // Clear operators
           decPlaces       := 1  
           mostDecPlaces   := 1
           beforePoint     := true     // Start with whole number entry
           display.text    := SPACE( MAX_DIGITS_DEC - 1 ) + "0"
      endwith   
   return ( this.form.display.text )

   function displayValue( value )
      //
      //  Display value without the pick character.
      //
      local num, pickLoc
      num = value
      pickLoc = at("&",num)

      do case
         case ( pickLoc <> 0 )
              num := stuff(num,pickLoc,1,"")
         case ( right(num,2) = " 0" )
              num := space(MAX_DIGITS_DEC)
         case ( left(num,1) = SPACE(1) )
              num := substr(num,2)
       endcase

   return ( num )

   Function FormCanClose
      If Type('Form.Save_Decimals') <> 'U'
         Set Decimals to  Form.Save_Decimals
      EndIf
      If Type ('Form.Save_CuaEnter') <> 'U'
      	If Form.Save_CuaEnter = 'ON'
		      Set CUAEnter On
         Else
         	Set CUAEnter Off
         EndIf
      EndIf
      //For my in-house use!
      If type('_app.ini') <> 'U'
      	Set Procedure to :dulfp:Center32.prg additive
         _app.ini.SetValue(id(),'Calculator',SetFrame(Form.Hwnd,0,0))
         _app.ini.Flush()
         Close Procedure :dulfp:Center32.prg
      EndIf
      Return True

   Function formOnClose
      //
      // Clean up
      //
      *CLOSE PROCEDURE PROGRAM(1)
   return //( true )

   function formOnGotFocus
      //
      // Make sure decimal places is correct every time 
      // this form gets focus.
      //
      If Type('Form.Save_Decimals') = 'U'
         Form.Save_Decimals = Setto('Decimals')
      EndIf

      If Type ('Form.Save_CuaEnter') = 'U'
	      Form.Save_CuaEnter =  Set('CuaEnter')
      EndIf
      Set CUAEnter On


      If Type('This.DecPlaces') = 'U'
         //Form onopen not yet processed
         Set Decimals to 1
      Else
         set decimals to this.decPlaces
      EndIf //Type('This.DecPlaces') = 'U'
      Form.Keyhandler.SetFocus()
   return //( SET("DECIMALS") )

   function keyHandlerOnChar(nChar, nRepCnt, nFlags)
      local sChar, vChar
      sChar = UPPER( CHR( nChar ) )
      vChar = VAL( sChar )
      if ( vChar > 0 AND vChar < this.form.numberSys )  OR ;
         ( sChar $ 'ABCDEF' AND form.numberSys = BASE_HEX)
         class::enterDigit( sChar )
      else
         do case
            case ( sChar == "0" )
                 class::enterDigit( sChar )
            case ( sChar == "/" )
                 class::enterOp( this.form.opDiv )
            case ( ( sChar == "*" ) or ;
                   ( sChar == "X" ) )
                 class::enterOp( this.form.opTimes )
            case ( sChar == "-" )
                 class::enterOp( this.form.opMinus )
            case ( sChar == "+" )
                 class::enterOp( this.form.opPlus )
            case ( sChar == "^" )
                 class::enterOp( this.form.opPower )
            case ( ( sChar == "=" ) or ;
                   ( nChar == key_enter  ) )
                 class::enterOp( this.form.opEqual )
            case ( ( sChar == SETTO("POINT") ) and ;
                   ( this.form.point.enabled ) )
                 class::pointOnClick()
            case ( nChar == Key_Escape )
                 class::BUTTONCLEARONCLICK()
            case ( nChar == KEY_BACKSPACE )
                 class::removeLastDigit()
         endcase
      endif
   return // (nChar)

   function removeLastDigit
      //
      // Remove last digit for backspace key
      //
      if ( not this.form.lastKeyOperator )
 
         if ( not ( class::numVal( this.form.display.text ) == 0 ) )
            this.form.display.text := ;
               this.form.display.text.substring( 0, this.form.display.text.length - 1 )
         endif

         if ( class::numVal( this.form.display.text ) == 0 )
            this.form.display.text := SPACE( MAX_DIGITS_DEC - 1 ) + "0"
         endif

      endif

   return true

   function McOnClick      
      //
      // Memory Clear
      //
      local thisForm
      if ( TYPE("this.form") == "O" )
         thisForm = this.form
      else
         thisForm = this
      endif
      with ( thisForm )
         lastKeyOperator     := true
         memory.value        := space(MAX_DIGITS_DEC - 1) + "0"
         textMIndicator.text := null
      endwith
   return ( thisForm.memory.value )

   function MrOnClick
      //
      // Recall from memory.
      //

      if ( this.form.lastKeyOperator )
         with ( this.form )
            lastKeyOperator := false
            beforePoint     := true    
         endwith
      endif
      this.form.lastValue    := class::numVal( this.form.display.text )
      this.form.display.text := this.form.memory.value
   return ( this.form.display.text )

   function formOpen

      If Type ('Form.Save_CuaEnter') = 'U'
	      Form.Save_CuaEnter =  Set('CuaEnter')
      EndIf
      Set CUAEnter On

      If Type('Form.Save_Decimals') = 'U'
         Form.Save_Decimals = Setto('Decimals')
      EndIf
      set decimals  to 1 

      // add clipboard entryField
      this.eClip = new EntryField(this)
      this.eClip.visible = false

      //Check to see if called from spinbox Custom Control
      If Type('Form.lLink') = 'U'
         Form.lLink = False
      EndIf

      //Set Paste Button to reflect status of Form.lLink
      If Form.lLink
         //Called from custom control - enable paste
         Form.Pastebutton.UpBitMap = "RESOURCE TS_PASTE"
         Form.PasteButton.Speedtip = "Paste Result To EntryField"
         Form.Pastebutton.OnClick = class::PASTEBUTTON_ONCLICK
//         Form.Root.EditMenu.Paste.Enabled = True
      Else
         //Not called from custom control -- enable copy
         Form.Pastebutton.UpBitMap = "RESOURCE TS_COPY"
         Form.PasteButton.Speedtip = "Copy Result To Windows ClipBoard"
         Form.Pastebutton.OnClick = class::copyToClip
//         Form.Root.EditMenu.Paste.Enabled = False
      EndIf //Form.lLink

      this.height     := 200             // hide hex 
      this.point.text := SETTO("POINT") // Match current country  "," or "."

      this.opPlus.doIt        = { |a,b| ; return ( a + b ) }
      this.opPlus.precedence  = 1

      this.opMinus.doIt       = { |a,b| ; return ( a - b ) }
      this.opMinus.precedence = 1

      this.opTimes.doIt       = { |a,b| ; return ( a * b ) }
      this.opTimes.precedence = 2

      this.opDiv.doIt         = class::opDivideIt
      this.opDiv.precedence   = 2

      this.opPower.doIt       = { |a,b| ; return ( a ^ b ) }
      this.opPower.precedence = 3

      this.opEqual.doIt       = { |a,b| ; return null }
      this.opEqual.precedence = 0

      this.mAdd.doIt          = { |a,b| ; return ( a + b ) }
      this.mAdd.precedence    = 1

      this.mSub.doIt          = { |a,b| ; return ( a - b ) }
      this.mSub.precedence    = 1

      //
      // Create custom form properties
      // 
      // Stack for storing operation states
      this.operationStack = new OperationStackClass()
      this.operationStack.Initialize() // Empty operation stack         
 
      this.numberSys = BASE_DEC 
      this.decPlaces      = 1    // Start with minimum decimal places
      this.mostDecPlaces  = 1    // Most decimal places in an operand

      this.beforePoint     = true // Add numbers before or after decimal pt.
      this.lastKeyOperator = null
      this.lastValue       = 0        
      If NOT Form.lLink
         //If not called from spinbox, set to zero
         this.display.text := SPACE( MAX_DIGITS_DEC - 1 ) + "0"
      EndIf //NOT Form.lLink
      class::mcOnClick()
      this.KEYHANDLER.setFocus()
      /*
      msgbox("L:" + str(this.left) + ;
      		 "T:" + str(this.TOP) +  ;
             "H:" + str(this.HEIGHT) +; 
             "W:" + str(this.WIDTH) +  ;
             "WS;" + str(this.WINDOWSTATE))
      */

   return //( CALCULATORFORM::OPEN() )

   function copyToClip
      Form.eClip.value := ;
         LTRIM( Form.display.text )
      Form.eClip.selectAll := true
      Form.eClip.setFocus()
      Form.eClip.copy() 
      Form.keyHandler.setFocus()
   return true

   Function Pastebutton_Onclick
   	If Form.NumberSys <> Base_Dec
      	If Msgbox('Convert result to decimal?','Please Verify', 32 + 4) = 6
         	//yes
            class::changeBase( BASE_DEC )
         Else
         	MsgBox('Cannot paste non-decimal number to spinbox.',;
                   'Abandoning changes', 64)
            form.Close(False)
            Return
         EndIf //Msgbox('Convert result to decimal?','Please Verify', 32 + 4) = 6
      EndIf //Form.NumberSys <> Base_Dec
      Form.Close(Val(LTRIM( Form.display.text )))

   function opDivideIt( a, b )
      local nResult
      nResult = 0
      if ( b == 0 )
         MSGBOX("Cannot divide by zero.", "Alert",48)
      else
         nResult := ( a / b )
      endif
   return ( nResult )

   function charVal( num )
      //
      // Convert decimal number to string using 
      // current base: hex, decimal, octal, binary.
      //
      local string, fractionVal
      local nPlace
      local nPValue
      string = ""

      do case
         case ( this.form.numberSys == BASE_HEX )

              if ( ABS(num) >= HEX_OVERFLOW )
                 string := replicate("*", MAX_DIGITS_HEX)
              else
                 string := itoh(num)
                 string := space(MAX_DIGITS_DEC - len(string)) + string
              endif

         case ( this.form.numberSys == BASE_OCT )
              string := class::toBaseString( num, 16, BASE_OCT )

         case ( this.form.numberSys == BASE_BIN )
              string := class::toBaseString( num, 16, BASE_BIN )

         case ( this.form.numberSys == BASE_DEC )
              if ( ABS(num) >= DEC_OVERFLOW )
                 string := replicate("*", MAX_DIGITS_DEC)
              else
                 string := class::decimalShowString( num )
              endif

      endcase
   return ( string )

   function decimalShowString( n )
      local nDec, sInt, sMask, sVal     
      nDec = SET("DECIMALS")
      sInt  = ""
      sMask = ""
      sVal  = ""
      SET DECIMALS TO 18

      if ( n < 0 ) and ( n > -1 )
         sInt := "-0"
      else
         sInt := int( n )+"".rightTrim()
      endif

      sMask := replicate("9", sInt.length ) + "." + ;
                 replicate("9", ( 14 - sInt.length ) )
      sVal := TRANSFORM(n,sMask)

      do while ( SETTO("POINT") $ sVal ) and ( sVal.right(1) == "0" )
         sVal := sVal.substring( 0, sVal.length - 1 )
      enddo

      if sVal.right(1) == SETTO("POINT")
         sVal := sVal.substring( 0, sVal.length - 1 )
      endif

      SET DECIMALS TO ( nDec )
   return ( sVal )

   function toBaseString( nDecValue, nMaxPlace, nBase )
      local nPlace
      local npValue
      local nAdd
      local sValue
      local bStart
      local bCarry

      nPlace  = nMaxPlace
      npValue = 0
      sValue  = ""
      bStart  = false
      if ( ABS( nDecValue ) >= nBase ^ nPlace )
         sValue = REPLICATE("*", nMaxPlace )
      else                
         do while ( nPlace >= 0 )
            if ( nDecValue == 0 )
               sValue := sValue + "0"
            else
               npValue := ( nBase ^ nPlace )
               if ( ( npValue <= nDecValue ) or ;
                    ( npValue == 1 ) )
                  bStart := true
                  if ( nDecValue / npValue ) == nBase
                     sValue := sValue + "10"
                  else
                     sValue := sValue + ;
                              ALLTRIM( STR( INT( nDecValue / npValue )  ) )
                  endif
                  nDecValue := MOD( nDecValue, npValue )
               else
                  if ( bStart )
                     sValue := sValue + "0"
                  endif
               endif
            endif
            nPlace := nPlace - 1
         enddo
         if ( VAL( sValue ) == 0 )
            sValue := "0"
         endif
         sValue := SPACE( MAX_DIGITS_DEC - LEN( sValue ) ) + sValue
      endif
   return ( sValue )

   function numVal( string )
      //
      // Return decimal value of display value using
      // current base to translate.
      //
      local h, num, nPoint
      local sValue
      local nDigit, nPlace
      private macro

      sValue  = ALLTRIM( string )
      h       = ""
      macro   = ""
      num     = 0
      nDigit  = 0
      nPlace  = 0
      nPoint  = 0

      do case

         case ( this.form.numberSys == BASE_HEX )

              h   := htoi(string)
              num := iif(h >= HEX_OVERFLOW/2, bitxor(h, HEX_OVERFLOW), h)

         case ( this.form.numberSys == BASE_OCT )

              if ( sValue == null )
                 num := 0
              else
                 if ( SUBSTR( sValue, 1, 1 ) == "*" )
                    num := ( 1 / 0 )
                 else
                    macro := "0" + sValue
                    num := &macro.
                 endif
              endif

         case ( this.form.numberSys == BASE_BIN )

              do while ( LEN( sValue ) > 0 )
                 nDigit := IIF( RIGHT( sValue, 1 ) == "1", 1, 0 )
                 sValue := SUBSTR( sValue, 1, ( LEN( sValue ) - 1 ) )
                 num    := num + ( nDigit * (2 ^ nPlace) )
                 nPlace := nPlace + 1
              enddo

         otherwise            
              // decimal
              nPoint := AT( SETTO("POINT"), sValue )

              if ( nPoint <> 0  )               
                 num := VAL( STUFF( sValue, nPoint, 1, SETTO("POINT") ) )
              else
                 num := VAL( sValue )
              endif

      endcase

   return ( num )
endclass
class OperationStackClass
   //
   // This class defines a stack object for storing 
   // mathematical operation states.  This is necessary 
   // for using operators that have different precedence, like
   // + and *.  This is a stack of OperationStateClass objects.
   //
   
   class::initialize()


   function initialize
      //
      // Create empty stack
      //

      // Bottom (and top, at first) of Stack
      this.stackTop = new OperationStateClass()
   return true

   function isEmpty
      //
      // Check if stack is empty
      //

   return empty( this.stackTop.getNext() ) // Empty if no more operations


   function pop
      //
      // Retrieve last operation state
      //
      local xOp
      xOp = false

      if ( not empty(this.stackTop.getNext()) )
         xOp := this.stackTop
         this.stackTop = this.stackTop.getNext()
      endif

   return xOp


   function push( operation )
      //
      // Save last operation state
      //
      local newStackTop

      newStackTop = new operationStateClass()  // Create new operationState
      newStackTop.assign( operation )          // Assign to it properties of
                                               // operation
      newStackTop.setNext( this.stackTop )     // Make current top be next

      this.stackTop = newStackTop              // Make new operation be top

   return true


   function prevPrecedenceGreaterOrEqual(curPrecedence)
      //
      // Retrieve last operation state
      //
   return ( this.stackTop.getPrecedence() >= curPrecedence )

endclass

class OperationStateClass
   //
   // This class defines a single operation state, which is to 
   // be stored on the operationStack
   //

   this.op         = { |a,b| ; return null } 
   this.precedence = -1              
   this.opValue    = 0
   this.next       = false

   function getOp
   return ( this.op )


   function setOp( newOp )
      this.op := newOp
   return ( this.op )


   function getPrecedence
   return ( this.precedence )


   function setPrecedence( newPrecedence )
      this.precedence := newPrecedence
   return ( this.precedence )


   function getValue
   return ( this.opValue )

   function setValue( newValue )
      this.opValue := newValue
   return ( this.opValue )


   function getNext
   return ( this.next )

   function setNext(newNext)
      this.next := newNext
   return ( this.next )

   function assign( newOperationState )
      with ( this )
         op         := newOperationState.doIt
         precedence := newOperationState.precedence
         opValue    := newOperationState.opValue
      endwith
   return ( this.opValue )

endclass



