/*
    File.......: dUFLP.CFM
    Author ... : Ken Mayer
    Date.......: January 7, 1999

    This custom form is aimed at giving the user of the dUFLP
    library something to start from, if they don't already have
    a custom form set up. You can always change whatever you 
    want here. 

    Default properties of this form which have been changed:
       autoCenter = true
       mdi        = false
       sizeable   = false
       moveable   = false
       maximize   = false
       minimize   = false
       text       = "dUFLP Custom Form"
       escExit    = false

       (this is an SDI form, you can change that by
        changing the MDI property to true -- note that
        the sizeable, moveable, maximize, minimize 
        properties will all be ignored if this is
        an MDI form ... when the form gets opened,
        if SDI open with READMODAL, if MDI open with OPEN())

    Default events of this form have been modified:
       onOpen     = code to set the rowset at the first row,
                    set the rowset's autoEdit property
                    to false, and attach the canNavigate
                    code in this form to the rowset's
                    canNavigate.

                    NOTE: if you are using multiple rowsets,
                    you will want to set the autoEdit
                    to false for those rowsets and may
                    wish to attach the canNavigate code
                    below to the canNavigate of those
                    rowsets:
                      form.datamodref1.ref.query2.rowset.canNavigate :=;
                         class::rowset_canNavigate
                       OR
                      form.query2.rowset.canNavigate :=
                         class::rowset_canNavigate
       canClose   = code to make sure the user doesn't
                    close the form while adding/editing a row

       Note that any derived forms will inherit these properties,
       so if you don't want them in all your forms that
       are derived from this, change them *here*.
    
    Custom controls on form (kmXXX are from the dUFLP custom control
    files): 
          Page 0: kmCustTabbox             (from FORMCNTL.CC)
          Page 1: kmCustVerticalTitle1     (from FORMCNTL.CC)
                  KensToolBar              (from CUSTBUTT.CC)
          Page 2: kmCustText               (from FORMCNTL.CC)
                  seeker                   (ships with product)
                  kmCustGrid              (from FORMCNTL.CC)

                  If you wish the functionality of 
                  "kmCustGrid" (ability to set grid-wide
                  font/color properties) change the reference
                  to "kmCustGrid" to simply "kmCustGrid".

    Special methods of this form:
       repaint() -- this can be used to ensure that 
                    the form has been repainted.
                    formreference.repaint()
                       or
                    formreference.repaint( anotherformreference )

                    i.e.:
                       set proc to myform.wfm addi
                       f = new myformForm()
                       f.open()
                         // do stuff
                       f.repaint()
                         or if using a child form:
                       f.repaint( childreference )
                         

       enableControls() --
                    Can be used to enable or disable controls
                    on a form. It is designed to be
                    recursive, so that if you have a container
                    (container or notebook objects) on the
                    form, you can enable or disable the controls
                    on that as well, and it was written so that
                    you can enable/disable controls ONLY
                    on a specific container object.

                    FORM.EnableControls( false ) // disable all controls
                    FORM.EnableControls( true )  // enable all controls
                    FORM.EnableControls( false, FORM.Notebook1 )
                                      // disable all controls ONLY
                                      // on Notebook1
                    FORM.EnableControls( true, FORM.Notebook1 )
                                      // enable all controls ONLY
                                      // on Notebook1
                    These last two commands can be used for a 
                    container as well.


       rowset_canNavigate --
                   attached to a rowset's canNavigate
                   event, this will ask the user if they
                   are a) in edit or append mode and b) have
                   modified the row, if they want to 
                   save or abandon the changes made,
                   or if they want to cancel the navigation.

    Usage:
       * Bring up a *new* form in the designer form designer
         (i.e., type: create form test   in the command
         window, or double-click the new form icon)
       * Go to the file menu
       * Select "Setup Custom Form"
       * Select dUFLP.CFM
       * Click "OK"


       * NOTE: If you have multiple rowsets on this form
         either with a datamodule or simply multiple queries,
         and you switch between them in some fashion (possibly
         using a notebook's onSelChange event), you 
         will want to make sure that the following is 
         executed each time you change rowsets (this
         changes the form's rowset reference):

         form.rowset := form.query2.rowset
         form.kensToolbar1.rowset := form.query2.rowset
         form.KensToolbar1.initRowset()

         or change "form.query2" to "form.datamodref1.ref.query2"
         if using a datamodule. 

    Other Misc Notes:
     * If you bring this form into the designer, every pushbutton
       on KensToolBar will have a "value" property placed
       into the custom form -- you can safely delete each code
       block that looks like:

         with (this.KENSTOOLBAR1.NAVIGATEBUTTONS.KMTOPBUTTON1)
            value = false
         endwith

       the "value" property for pushbuttons isn't being used
       in any of these buttons ... (these properties being
       streamed out is a known bug in the designer) 
       The same will occur for the rowState control on KensToolBar
       as well ...

       This will also occur for any derived form that you 
       bring into the designer. If you don't change the
       properties of any of these buttons, then you can safely
       delete the codeblocks as shown above in the derived
       forms as well.

     * When deploying an application that uses this form,make sure
       that the project manager includes references to each of the
       custom control files and this file:
          duflp.cfm
          seeker.cc 
              (will be in your 
               drive:\Program Files\dBASE\dB2K 01\CLASSES
               directory)
          custbutt.cc
          formcntl.cc

     * There is some code added here to ensure that when the user
       attempts to close the form ('x' in the title bar, 
       Ctrl+F4, etc.) that if the user is in the middle of
       editing/adding a new row, they are prompted before
       leaving it ...

     * Any form derived from this one that has a rowset
       assigned to the form's rowset property will have the
       rowset's autoEdit property set to false, and the
       canNavigate event will point to the code below 
       (rowset_canNavigate).

   ** Note that this form uses Source Aliasing, specifically the
   ** dUFLP source Alias. If you do not know how to set up
   ** a Source Alias, see the instructions in WHATS.NEW at
   ** the top of the file, or in README.TXT.
   ** Any files referenced by :dUFLP:filename  must be
   ** included in an executable if you are building 
   ** one ... make sure that your project includes these
   ** files.
   **
   ** This particular form uses:
   **     custbutt.cc
   **     formcntl.cc
   **
   ** From the :classes: source alias, this uses
   ** seeker.cc -- note that "classes" is defined
   ** by the install of dB2K

   ** When creating a form derived from this, make sure
   ** that you use:
   **  class MyForm of dUFLPCForm from ":dUFLP:dUFLP.CFM"

   // special note -- in dBASE Plus, the :classes: source
   // alias is redefined as :formcontrols:  the following
   // code has been added to the constructor code of the
   // form, but if you load this in the form designer and change
   // anything, saving the form will probably cause the following
   // code to be stripped out. You may want to check and see.
   // In addition, running using or compiling this custom form in 
   // dB2K or in dBASE Plus, you will want to re-compile it in the 
   // other or it won't run ...
   #if __version__ < 2
      set procedure to :classes:SEEKER.CC additive
   #else
      set procedure to :formcontrols:SEEKER.CC additive
   #endif

*/

class duflpCForm of FORM custom
   set procedure to :dUFLP:Custbutt.cc additive
   #if __version__ < 2
      set procedure to :classes:SEEKER.CC additive
   #else
      set procedure to :formcontrols:SEEKER.CC additive
   #endif
   set procedure to :dUFLP:formcntl.cc additive
   with (this)
      canClose = class::FORM_CANCLOSE
      onOpen = class::FORM_ONOPEN
      scaleFontBold = false
      height = 16.1364
      left = 23.7143
      top = 3.2273
      width = 70.7143
      text = "dUFLP Custom Form"
      autoCenter = true
      mdi = false
      sizeable = false
      moveable = false
      escExit = false
      maximize = false
      minimize = false
   endwith


   this.KMFORMTITLE1 = new KMFORMTITLE(this)
   with (this.KMFORMTITLE1)
      height = 1.3182
      left = 0
      top = 0
      width = 70.7143
      anchor = 2	// Top
      text = "This Form Does Something"
   endwith


   this.KMVERTICALFORMTITLE11 = new KMVERTICALFORMTITLE1(this)
   with (this.KMVERTICALFORMTITLE11)
      height = 13.7273
      left = 64.1429
      top = 1.3182
      width = 6.5714
      anchor = 0	// None
      text = "dUFLP Custom Form"
   endwith


   this.KENSTOOLBAR1 = new KENSTOOLBAR(this)
   with (this.KENSTOOLBAR1)
      left = 0.4286
      top = 1.3636
      width = 60.1429
      height = 1.3182
   endwith

   this.KMCUSTTABBOX1 = new KMCUSTTABBOX(this)
   with (this.KMCUSTTABBOX1)
      height = 1
      left = 0
      top = 15.1364
      width = 70.7143
   endwith


   this.SEEKERTEXT = new KMCUSTLABEL(this)
   with (this.SEEKERTEXT)
      height = 0.9545
      left = 5.8571
      top = 0.4091
      width = 6.7143
      text = "Find:"
      pageno = 2
   endwith


   this.SEEKER1 = new SEEKER(this)
   with (this.SEEKER1)
      height = 1
      left = 13.1429
      top = 0.3636
      width = 43.1429
      value = ""
      pageno = 2
   endwith


   this.kmCustGrid1 = new kmCustGrid(this)
   with (this.kmCustGrid1)

      pageno = 2
      height = 13
      left = 0.5714
      top = 1.6818
      width = 69.2857
   endwith


   function Repaint
      /*
         This is a routine by Jim Sare with minor
         modifications by Ken Mayer. It can be used
         to ensure that if any processing has occurred
         that the form gets repainted (which sometimes
         doesn't happen) ...
      */
      parameter oForm
      if type( "oForm" ) == "U" or empty( oForm )
         oForm = form
      endif
      if type( "UpdateWindow" ) # "FP"
         extern CLOGICAL UpdateWindow( CHANDLE ) USER32
      endif
   RETURN UpdateWindow( oForm.hWnd )

   PROCEDURE EnableControls
   /*
        Enable Controls -- this will loop through all controls
        on a form and set their enabled property to true or false
        depending on the parameters you pass it.

        Author: Jim Sare
        Posted on the VdBASE Newsgroups, 7/9/1998

        Usage Notes:

           This particular method enables/disables all controls in the 
           form and any containers on the form. It can be adapted to
           other purposes.

           FORM.EnableControls( false ) // disable all controls
           FORM.EnableControls( true )  // enable all controls
           FORM.EnableControls( false, FORM.Notebook1 )
                                        // disable all controls ONLY
                                        // on Notebook1
           FORM.EnableControls( true, FORM.Notebook1 )
                                        // enable all controls ONLY
                                        // on Notebook1
           These last two commands can be used for a container as well.

           The function is recursive so if a container contains
           controls and another container or notebook, the controls
           contained in those containers and/or notebooks will also
           be affected.

   */
      parameters lEnable, o
      private oRef
      local oParent

      oParent = iif( type( "o" ) == "O", o, FORM )
      oRef = oParent.first
      do
         if type( "oRef.first" ) == "O" 
            form.EnableControls( lEnable, oRef )
         endif
         if type( "oRef.Enabled" ) # "U"
            oRef.Enabled:= lEnable
         endif
         oRef = oRef.before
      until oRef.Name == oParent.first.name
    return
    // End of EnableControls

   function form_onOpen
      // default is to move to the top of the rowset 
      // when the form opens:
      if form.rowset # null
         // don't allow the user to edit the rowset unless
         // they click the edit button, or the append
         // button ...
         form.rowset.autoEdit    := false
         // don't allow the user to navigate off the 
         // row if it is being edited or added, without
         // saving or abandoning ...
         form.rowset.canNavigate := class::rowset_canNavigate
         // go to the top of the rowset
         form.rowset.first()
      endif
   return

   function form_canClose
      // code from Gary White -- this deals with the user
      // trying to close the form while in edit or append
      // mode and prompts them to save, abandon, or cancel
      // see the header for other ...
      if form.rowset # NULL and;
         ( form.rowset.state == 2 or form.rowset.state == 3 )
         if form.rowset.modified
            nAnswer = msgbox("Save changes before leaving record?", "Data has changed",32+3)
            do case
               case nAnswer == 6
                  form.rowset.save()
               case nAnswer == 7
                  form.rowset.abandon()
               otherwise
                  return false
            endcase
         endif
      endif
   return true

   function rowset_canNavigate
      // For whatever reason, if the user chooses to cancel 
      // the navigation this routine gets called twice.  For 
      // that reason, we set a flag to indicate it's been 
      // here once.  If the flag exists, we null it, if
      // not we run the validation. (Code by Gary White)
      local bYes, nAnswer
      if type("_app.cancelNav") == "L" and _app.cancelNav
         _app.cancelNav := null
         bYes = false
      else
         bYes = true
         if this.modified
            nAnswer = msgbox("Save changes before leaving record?", ;
                             "Data has changed",32+3)
            do case
               case nAnswer == 6
                  this.save()
               case nAnswer == 7
                  this.abandon()
               otherwise
                  bYes := false
                  _app.cancelNav = true
            endcase
         endif
      endif
   return bYes
endclass
/*
   End of File: dUFLP.CFM
*/


