/*
   ----------------------------------------------------------------
   Description:

      Send e-mail using Outlook Express (IE4 or Higher)
      Supports attachments.

      The class creates a properly formatted disk file with an EML 
      extension. It then runs the EML file which calls up 
      Outlook Express.  

      Full documentation for a properly formatted e-mail message can be 
      found in RFC 2045 and RFC 822
      http://www.faqs.org/rfcs/rfc2045.html
      http://www.faqs.org/rfcs/rfc822.html
      
   Programmers: 

      Adapted for dBASE by Michael Nuwer from Visual Basic code 
      by Guillermo F. Zambianch. Modifications made by Rich Muller 
      and Marc Hamelin

   History: 

      19 May 2003  Michael Nuwer posted the first version in dBASE code lib
       3 Oct 2003  Rich from Autotraker posted a way to use Windows scripting
                   that would automatically close Outlook Express.
       5 Dec 2003  Marc Hamelin merged Rich's code with the first version
                   code and added an autoSend property.
                               

   Usage: 

      EmailMsg = new Email()
      EmailMsg.Dest = "username@domain.ext"
      EmailMsg.Subject = "Title / Subject"
      EmailMsg.Message = "Enter your message here..."
      EmailMsg.Attachments.add("FileName1.ext")
      EmailMsg.Attachments.add("FileName2.ext")
      EmailMsg.Attachments.add("FileName3.ext")
      EmailMsg.Send()

   Properties:

      Dest      A string that contains the e-mail address for the To: field.
                valid formats are:
                  "John@somewhere.com"
                  "John Brown <John@somewhere.com>"
                  "John@somwhere.com;Mike@nowhere.com"

                Multiple addresses are separated with a semicolon (;) 
                or with a coma (,)
                The class contains no code for validating the format of
                an e-mail address.

      ccDest    A string that contains the e-mail address(es) for the cc: field
      bccDest   A string that contains the e-mail address(es) for the bcc: field
      Subject   A string that contains the message subject 
      Message   A string that contains the message content
                The message is sent "as is," so if you want a multiple line
                use chr(13) + chr(10) at the end of line 
      Attachments  An array that contains the path and file name(s) of each
                   attachment to add to the message 

      AutoSend  A boolean. If true the Outlook Express window will 
                automatically send the message and close. If false the 
                user must manually send the message and close the window.
                 
      charset   A string that contains the character set to be used by the
                e-mail client, e.g. "windows-1252" (default) or "
                iso-8859-2" or "windows-1250"
      

*/
Class Email


   this.Dest = ""     // To:
   this.ccDest = ""   // Cc:
   this.bccDest = ""  // Bcc:
   this.Subject = ""  // Subject
   this.Message = ""  // Message Content  [end-of-line: chr(13) + chr(10)]
   this.AutoSend = true
   this.Attachments = new Array()
   this.charset = "windows-1252"   // or "iso-8859-2" or "windows-1250"
   this.bnd = "_NextPart_001_"     // boundary

   function Send
      local i, x, f, g, FileName

      f = new file()
      * Open temporary file
      FileName = getenv("temp") + "\" + funique("~???????.eml")
      f.create(FileName, "RW")
      * Write to temporary file
      *------------------------------------------------
      *  Recipients
      if NOT empty(this.Dest)
         f.puts("To: " + this.Dest)
      else
         this.AutoSend = false
      endif
      if NOT empty(this.ccDest)
         f.puts("Cc: " + this.ccDest)
      endif
      if NOT empty(this.bccDest)
         f.puts("Bcc: " + this.bccDest)
      endif
      *------------------------------------------------
      * Subject
      f.puts("Subject: " + this.Subject)
      *------------------------------------------------
      f.puts("MIME-Version: 1.0" + chr(13) + chr(10) + ;
               "Content-Type:multipart/mixed;")
      f.puts(chr(9) + "boundary=" + ["] + this.bnd + ["])
      f.puts("X-Unsent: 1")
      f.puts("")
      f.puts("")
      f.puts("--" + this.bnd)
      f.puts("Content-Type: text/plain;")
      f.puts(chr(9) + "charset=" + ["] + this.charset + ["])
      f.puts("Content-Transfer-Encoding: 7bit")
      f.puts("")
      *------------------------------------------------
      * Text message - Msg body
      f.puts(this.Message)
      f.puts("")
      *------------------------------------------------
      * Add Attachs

      LineLength = 76
      for i = 1 to this.Attachments.size
         att = this.Attachments[i]
         cAttName = substr(att, rat("\",att)+1)
         f.puts("--" + this.bnd)
         f.puts("Content-Type: text/plain;" )
         f.puts(chr(9) + "name=" + '"' + att + '"')
         f.puts("Content-Transfer-Encoding: 7bit")
         f.puts("Content-Disposition: attachment;")
         f.puts(chr(9) + "filename=" + '"' + cAttName + '"')
         f.puts("")
         f.close()
         f.open(FileName, "AB")   // Append binary

         g = new file()
         fSize = g.size(att)
         g.open(att, "RB") // Read Binary

         lines = int(fSize / LineLength)
         for x = 1 To lines
            linea = g.read(LineLength)
            f.write(linea)
         next
         // remainder
         linea = g.read(fSize - lines * LineLength)
         f.write(linea)
         g.close()
         f.close()
         f.open(FileName, "A") // For Append
         f.puts("")
      next
      *------------------------------------------------
      f.puts("--" + this.bnd + "--")
      * Close temporary file
      f.close()

      #define SW_SHOWNORMAL 1
      #define WINAPI_SHELL "SHELL32"
      #define WINAPI_A "A"

      if type("ShellExecute") # "FP"
         extern cHandle ShellExecute(cHandle, cString, cString, cString, ;
              cString, cINT) WINAPI_SHELL from "ShellExecute"+WINAPI_A
      endif
      ShellExecute(null, "open", FileName, null, null, SW_SHOWNORMAL)
      * if default e-mail program is Outlook Express,
      * this will show "New Message" window
      *------------------------------------------------

      *------------------------------------------------
      * Automatically sends the email if the AutoSend
      * option is set to true
      if this.AutoSend == true
         Start = dateTime().getTime() / 1000
         oShell = new OleAutoClient("WScript.Shell")
         do while true
            if oShell.AppActivate(this.Subject) OR ;
                 ((dateTime().getTime() / 1000) - Start) > 20
               exit
            endif
         enddo
         if oShell.AppActivate(this.Subject)
            oShell.SendKeys("^~") // Ctrl+Enter
         endif
         release object oShell
         oShell = null
         f.delete(FileName) // Delete the temporary email file
      endif
      *------------------------------------------------

      return

endclass
