/*
   Class: ImageNotebook
   Author: Daniel Howard (sproket@total.net)

   Inspired by : Timbert Kriek and Romain Strieff

   Version 1.0 May 23, 1999
           1.1 May 24, 1999 - added raised borderstyle for selected tab

   Properties:

   aDataSource       - An array pointer to the datasource 

   originalDataSource- A copy of the datasource since I destroy
                       it to make my own.

   aTabWidth         - An array I create which is 2 chars long which 
                       assign as the new datasource to the notebook 
                       so I can have a consistent tab width.

   aImages           - An Array of image objects which get created at
                       the form level to cover each tab. Each image
                       gets a custom Index property which is the curSel
                       of the notebook. I also add an Owner propery to 
                       it so you can reference the notebook itself.

   Methods: 

      Init           - Initializes the control. Requires dFORM or Peter
                       Rorlick's zForm.

      Image_onLeftMouseDown -
                        Used to change the page on the notebook.

   Notes:

      To define your images simply assign them to the datasource array.

      eg
      dataSource = 'ARRAY {"resource #600","resource #601","resource #610","resource #611"}'
      See the sample form as an example. 


      I didn't spend much time on the positioning logic but it looks 
      fairly good on my machine. You might have to tweek the code a 
      bit if you use a different standard font.

      This will not work with a scrolling notebook. All of the tabs have
      to be visible. 
*/

class ImageNOTEBOOK(parentObj) of NOTEBOOK(parentObj) custom
   with (this)
      height = 7.5
      left = 7
      top = 3.5
      width = 28
      metric = 0	// Chars
      dataSource = 'ARRAY {"First Tab", "Second Tab"}'
   endwith

   this.aDataSource = New Array()
   this.originalDataSource = this.dataSource
   this.aTabWidth = new array()
   this.aImages = new array()
   this.Cursel = 1

   // Create image objects on this notebook an position
   // them accordingly. + Create aDatasource property.
   Function Init

      // Create a reference to the datasource array
      private m_macro
      m_macro = substr(this.dataSource,6,len(this.dataSource))
      this.aDataSource := &m_macro.

      local j,nLeft
      nLeft = 0.60
      for j = 1 to this.aDataSource.Size
         this.aTabWidth.Add(str(j,2))
         this.aImages.Add( new Image(form) )         
         this.aImages[j].Top = this.Top + 0.2
         this.aImages[j].Left = this.Left + nLeft
         this.aImages[j].Height = 0.8
         if j = 1
            this.aImages[j].Width = 7
         else
            this.aImages[j].Width = 7
         endif
         nLeft += 8.2

         this.aImages[j].onLeftMouseDown = Class::Image_OnLeftMouseDown
         this.aImages[j].borderStyle := 3
         this.aImages[j].alignment := 3
         if "#" $ this.aDataSource[j]
            this.aImages[j].dataSource := this.aDataSource[j]
         else
            this.aImages[j].dataSource := "fileName "+ this.aDataSource[j]
         endif
         this.aImages[j].Index = j
         this.aImages[j].Owner = this
      next j

      this.aImages[1].BorderStyle := 1 // raised
      this.DataSource := "Array this.aTabWidth"
   return

   Function Image_onLeftMouseDown(flags, col, row)
      if this.Owner.Cursel # this.Index
         this.Owner.aImages[this.Owner.Cursel].BorderStyle = 3
      endif
      this.Owner.CurSel := this.Index
      this.BorderStyle = 1
   return

endclass



/*
   Class: ImageNotebook
   Author: Daniel Howard (sproket@total.net)

   Inspired by : Glenn Johansen

   Version 1.0 May 23, 1999
           1.1 May 24, 1999 - added raised borderstyle for selected tab

   Properties:

   aDataSource       - An array pointer to the datasource 

   aTexts            - An Array of text objects which get created at
                       the form level to cover each tab. Each text
                       gets a custom Index property which is the curSel
                       of the notebook. I also add an Owner propery to 
                       it so you can reference the notebook itself.

   Methods: 

      Init           - Initializes the control. Requires dFORM or Peter
                       Rorlick's zForm.

      Text_onLeftMouseDown -
                        Used to change the page on the notebook.

   Notes:

      I didn't spend much time on the positioning logic but it looks 
      fairly good on my machine. You might have to tweek the code a 
      bit if you use a different standard font.

      This will not work with a scrolling notebook. All of the tabs have
      to be visible. 
*/
class ColorTabNOTEBOOK(parentObj) of NOTEBOOK(parentObj) custom
   with (this)
      height = 7.5
      left = 7
      top = 3.5
      width = 28
      metric = 0	// Chars
      dataSource = 'ARRAY {"First Tab", "Second Tab"}'
   endwith

   this.aDataSource = New Array()
   this.TextColor = "white/red"
   this.aTexts = new array()
   this.Cursel = 1

   // Create text objects on this notebook an position
   // them accordingly. + Create aDatasource property.
   Function Init

      // Create a reference to the datasource array
      private m_macro
      m_macro = substr(this.dataSource,6,len(this.dataSource))
      this.aDataSource := &m_macro.

      local j,nLeft,nWidth
      nLeft = 0.80
      for j = 1 to this.aDataSource.Size
         this.aTexts.Add( new Text(form) )
         this.aTexts[j].Text := this.aDataSource[j]

         nWidth = this.aTexts[j].getTextExtent(this.aDataSource[j])
         this.aTexts[j].Width := nWidth + 0.5

         this.aTexts[j].Left := this.Left + nLeft
         this.aTexts[j].Top := this.Top + 0.2

         if type("this.TextColor") = "A"
            this.aTexts[j].ColorNormal := this.TextColor[j]
         else
            this.aTexts[j].ColorNormal := this.TextColor
         endif

         this.aTexts[j].onLeftMouseDown = Class::Text_onLeftMouseDown
         this.aTexts[j].Owner = this
         this.aTexts[j].Index = j
         nLeft += nWidth + 1.55
      next j
      this.aTexts[1].BorderStyle := 1 // raised

   return

   Function Text_onLeftMouseDown(flags, col, row)
      if this.Owner.Cursel # this.Index
         this.Owner.aTexts[this.Owner.Cursel].BorderStyle = 3
      endif
      this.Owner.CurSel := this.Index
      this.BorderStyle = 1
   return

endclass
