/* Class DateRange
	Pick a range of dates with the help of the mouse

   Programmer 	: Marc VdB
   Date			: 02-03-2009
   Dependencies: none

   Version 1.01 : thanks to Glenn Johansen , one can select a datespan
   					going back in time...
           1.02 : thanks to John Marshal , the starting day of the week 
           			can be set as third parameter of the constructor
           1.03 : Bugfix - the reverse selection had a problem to correctly
           			extract the starting date.
           1.04 : The first day of the month is drawn, so that a part of the 
           			previous month is always visible.
           1.05 : Added a property : selectable
           			if this property is true, one can start a selection at a date
                  that is not in the current month. Default = false
           1.06 : Bugfix - the daterange was not drawn correctly in some 
           			circumstances, when a carry month was involved.
           1.07 : Added a property "marktoday" if this is set to true,
           			Today is painted in the color "todaycolor" (default = false)
           1.08 : Changed the design of the constructor : the parameters are 
           			not passed in the constructor anymore, but are set by assigning
                  them to the appropriate properties (see demoform)
           1.09 : Added a property showbuttons , if this is set to true, three
           			supplemental buttons are displayed in the container, one can 
                  assign own onaccept and oncancel eventhandlers.
                  The Undorange clears the actual range and resets it to nothing
                  or to the preset range... (default = false)
           1.10 : Two new properties : preset_start and preset_end if these are 
           			not empty, the container will show up with the range highlighted
           1.11 : Current month is diplayed as abbreviation, code cleanup for the 
           			duflp, added some comments.

	Usage : (see also the demoform : datespantest.wfm)

   		   this.DC1 = new DATERANGE_CONTAINER(this)
   			with (this.DC1)
      			left = 42.0
			      top = 3.0
			   endwith

            and in the onopen of the form you can set the different properties


      this.dc1.startday    = 6					// first day of week is Sunday
      this.dc1.marktoday   = true            // highlight today
      this.dc1.selectable  = true            // select a date in a different month
      this.dc1.showbuttons = true            // show Cancel,Undo,Accept
      this.dc1.preset_start= new date(2009,2,20)   // start the control with
      this.dc1.preset_end  = new date(2009,3,2)		// a highlighted datespan

      * you can assign your own eventhandlers, to react on user input
      * onstartdate is fired, when the users clicks on a date
      * onenddate is fired when the mousebutton goes up on the enddate
      * onchangedate is fired during mousemove over a date
      * onaccept and oncancel fire, when the appropriate button is pressed

      this.dc1.onstartdate = class::write_start
      this.dc1.onenddate   = class::write_end
      this.dc1.onchangedate= class::write_end
      this.dc1.onaccept		= { ; Msgbox("You accepted the range","INFO",0) }
      this.dc1.oncancel		= { ; ? "CANCEL" }


*/

#define 	normaldatecolor	"BtnText/white"      // color of the date in the current month
#define	selectcolor			"BtnText/red"			// color for the highlighting
#define  othermonthcolor	"BtnText/silver"     // color for the dates not in current month
#define  todaycolor			"BtnText/green"		// color for the today-date

#define	weekdays_ge			{"Mo","Di","Mi","Do","Fr","Sa","So"}   // Abbreviation for the day
#define  weekdays_en			{"Mo","Tu","We","Th","Fr","Sa","Su"}	// of the week in different
#define  weekdays_fr			{"Lu","Ma","Me","Je","Ve","Sa","Di"}	// languages

Class DateRange_Container(oparent) of Container(oparent) custom

   this.onleftmouseup	= class::mouse_up
	this.onopen = class::startpoint     // set the values for the 42 textlabels
	this.metric				= 6
   this.days   			= new array(42)      // contains the textlabels
   this.reversedays		= new assocarray()	// lookup-array
   this.labels				= new array(7)       // header, for the days of week
   this.height 			= 186
   this.width				= 276
   this.borderstyle		= 1
   this.daylabels			= weekdays_ge  // change this to _en or _fr for other languages
   this.lmd			   	= false
   this.pstart 			= 0
   this.pakt				= 0
   this.pfirst				= 0
   this.pmfirst			= ""
   this.pdfirst			= ""
   this.selectable		= false
   this.marktoday			= false
   this.startday			= 0
   this.showbuttons		= false
   this.preset_start		= ""
   this.preset_end 		= ""
	this.current_date 	= date()
   this.starting_date	= new date()         // holds the selected start and end-date
   this.end_date			= new date()

	this.init()          // draw the textlabels (done in the constructor to avoid flickering)
   this.extract_current()   

   this.pb2 = new PUSHBUTTON(this)
   with (this.pb2)
   	onclick = { ; this.parent.change_month(0) }
      onmouseover = class::carry_month
      height =150
      left 	= 244
      top 	= 32
      width = 28
      text = ""
      upBitmap = "RESOURCE #100"
      speedbar = true
   endwith

   this.pb1 = new PUSHBUTTON(this)
   with (this.pb1)
   	onclick = { ; this.parent.change_month(2) }
      onmouseover = class::carry_month
      height =150
      left 	= 2
      top 	= 32
      width = 28
      text = ""
      upBitmap = "RESOURCE #104"
      speedbar = true
   endwith

   this.tl1 = new TEXTLABEL(this)
   with (this.tl1)
      height = 22.0
      left = 2
      top = 6
      width = 26
      text = ""
      alignHorizontal = 1
   endwith

   this.tl2 = new TEXTLABEL(this)
   with (this.tl2)
      height = 22.0
      left = 244.0
      top = 6
      width = 28
      text = ""
      alignHorizontal = 1
   endwith

   this.accept_button = new PUSHBUTTON(this)
   with (this.accept_button)
   	onclick = { ; this.parent.onaccept() }
      height = 24.0
      left = 3
      top = 183.0
      width = 89
      text = "Accept"
   endwith

   this.undobutton = new PUSHBUTTON(this)
   with (this.undobutton)
   	onclick = {; this.parent.preset_range() }
      height = 24.0
      left = 93
      top = 183.0
      width = 89
      text = "Undo"
   endwith

   this.cancelbutton = new PUSHBUTTON(this)
   with (this.cancelbutton)
   	onclick = { ; this.parent.oncancel() }
      height = 24.0
      left = 183
      top = 183.0
      width = 88
      text = "Cancel"
   endwith

   **End of constructor

   ** get the current year, month and day from the current date

   function extract_current()
      this.current_month   = month(this.current_date)
      this.current_year		= year(this.current_date)
   	this.current_day		= day(this.current_date)
   return

   ** if preset_start AND preset_end are not empty, the following 
   ** method draws a selection corresponding to these two dates

   function preset_range()
   	local tmp

      if (not empty(this.preset_start)) and (not empty(this.preset_end))
      	if dtos(this.preset_start) >= dtos(this.preset_end)
      		tmp = this.preset_start
         	this.preset_end = this.preset_start
         	this.preset_start = tmp
         endif
      	this.current_date = this.preset_start
         this.extract_current()
         this.get_display_parameters()
         this.pstart = this.reversedays[dtos(this.preset_start)]
         if this.reversedays.iskey(dtos(this.preset_end))
         	this.pakt = this.reversedays[dtos(this.preset_end)]
         else
         	this.pakt = 42
         endif
         this.dcolor()
         this.starting_date = this.preset_start
         this.end_date = this.preset_end
         this.onchangedate()
      else
      	this.get_display_parameters()
      endif
   return

   ** in the onopen of the container, set the day of week labels
   ** and the height (in order to show the buttons or not)

   function startpoint
   	local i
      for i = 0 to 6
      	this.labels[i+1].text = this.daylabels[1+(i+this.startday)%7]
   	endfor
      this.preset_range()
      if this.showbuttons = true
      	this.height = 211
      endif
   return

   ** helper function

   function make_ym(d_first)
   	local tmp
		tmp = str(year(d_first),4) + right("00"+month(d_first),2)
   return tmp

   ** if the cursor moves over the "next-month-button" while the 
   ** mousebutton is hold down, the next month is displayed and 
   ** the user can continue to select the end-date

   function carry_month
   	local pcont,offset
      pcont = this.parent
      if pcont.lmd
      	m_dir = iif(upper(this.name) == "PB2" , 0 , 2)
         if m_dir = 0 
            pcont.end_date = pcont.days[42].date
         else
            pcont.starting_date = pcont.days[1].date
         endif
      	pcont.change_month(m_dir)
         if m_dir = 0
         	if pcont.reversedays.iskey(dtos(pcont.starting_date))
            	pcont.pstart = pcont.reversedays[dtos(pcont.starting_date)]
            else
            	pcont.pstart = 1
            endif
            pcont.pakt = pcont.reversedays[dtos(pcont.end_date)]
            offset = pcont.pakt       	
         else
         	if pcont.reversedays.iskey(dtos(pcont.end_date))
            	pcont.pstart = pcont.reversedays[dtos(pcont.end_date)]
            else
            	pcont.pstart = 42
            endif 
           	pcont.pakt = pcont.reversedays[dtos(pcont.starting_date)] 
            offset = pcont.pakt
         endif
         pcont.set_cursorpos(offset)
      endif   
   return

   ** externalize the needed functions
   ** and display the textlabels for the header and the days themselves

   function init
      extern chandle 	SetCapture(chandle) user32
      extern clogical 	ReleaseCapture(cvoid) user32
      extern clogical 	SetCursorPos(cint,cint) user32
      extern clogical 	GetWindowRect(chandle,cptr) user32

   	local i,j,cnt,ctmp
      cnt = 1

      ** draw 6 rows a 7 days

		for i = 0 to 5
      	for j = 0 to 6
         	this.days[cnt] = new tdate(this,33+j*30,32+i*25,cnt)
            cnt++
         endfor
      endfor

      ** draw the header (abbreviation of the days)

      for i = 0 to 6
      	this.labels[i+1] = new tdaylabel(this,3+(i+1)*30," ")
      endfor
   return

   ** if the today-date is visible in the container, make sure that it
   ** is shown in the correct color

   function settoday
      if this.reversedays.iskey(dtos(date())) and this.marktoday = true
      	this.days[this.reversedays[dtos(date())]].colornormal = todaycolor
      endif
   return

   ** if we change the month, with mouse-button hold down,
   ** make sure that the user can continue to drag at the 
   ** correct position

   function set_cursorpos(n)
   	local buffer,x,y
      buffer = replicate(chr(0),8)
      getwindowrect(this.days[n].hwnd,buffer)
      x = 16777216*buffer.getbyte(3)+65536*buffer.getbyte(2)
     	x+= 256*buffer.getbyte(1)+buffer.getbyte(0) + 12
      y= 16777216*buffer.getbyte(7)+65536*buffer.getbyte(6)
     	y+= 256*buffer.getbyte(5)+buffer.getbyte(4) + 12
      setcursorpos(x,y)
   return

   ** set the colors and texts of the daylabels according to 
   ** the current month
   ** this method is called, each time the month changes

   function get_display_parameters

      local ofss,prevmon,i,j
   	
      ** get offset for current month

      ofss	=  dow(new date(this.current_year,this.current_month-1,1))-this.startday+7
   	this.offset_start= (5+ofss)%7 
      if this.offset_start = 0
      	this.offset_start = 7
      endif
      this.month_len = new date(this.current_year,this.current_month,1) - ;
      					  new date(this.current_year,this.current_month-1,1)
      this.offset_end = this.offset_start + this.month_len
		prevmon = day(new date(this.current_year,this.current_month-1,1)-1)-this.offset_start 

      ** clear the lookup-array

      this.reversedays.removeall() 

      ** set the values for the actual month
           
      for i = 1 to this.month_len
      	this.days[i+this.offset_start].text  = i
         this.days[i+this.offset_start].fontbold  = true
         this.days[i+this.offset_start].vcolor = normaldatecolor
         this.days[i+this.offset_start].date = new date(this.current_year,this.current_month-1,i)
         this.reversedays[dtos(this.days[i+this.offset_start].date)] = i+this.offset_start
      endfor

      ** set the values for the visible part of the previous month

      for i = 1 to this.offset_start
      	this.days[i].vcolor = othermonthcolor
         this.days[i].fontbold  = false
         this.days[i].text  = i+prevmon
         this.days[i].date = new date(this.current_year,this.current_month-2,i+prevmon)
         this.reversedays[dtos(this.days[i].date)] = i
      endfor

      ** set the values for the visible part of the next month

      for i = 1 to 42 - this.offset_end
      	this.days[i+this.offset_end].vcolor = othermonthcolor
         this.days[i+this.offset_end].fontbold  = false
         this.days[i+this.offset_end].text  = i
         this.days[i+this.offset_end].date = new date(this.current_year,this.current_month,i)
         this.reversedays[dtos(this.days[i+this.offset_end].date)] = i+this.offset_end
      endfor

      ** set the text for the actual month and year

      this.tl1.text = left(cmonth(this.current_date),3)
      this.tl2.text = str(this.current_year,4)
      this.draw(1,42)
      this.settoday()
   return

   ** explicit change of the actual month, when the corresponding buttons 
   ** are clicked

   function change_month(ndelta)
   	this.current_date = new date(this.current_year,this.current_month-ndelta,1)
      this.extract_current()
      this.get_display_parameters()
   return

   ** this method is used to display the standard-colors for the non selected
   ** days

   function draw(nstart,nend)
   	local i
      for i= nstart to nend
      	this.days[i].set_vdata()
      endfor
   return

   ** this method draws all selected days in the selectcolor color

   function dcolor
   	local i,n_start,n_end,n_delta
     	this.draw(1,42)
      n_start  = min(this.pakt , this.pstart)
      n_end		= max(this.pstart , this.pakt)
   	for i = n_start to n_end
      	this.days[i].colornormal = selectcolor
      endfor
		this.settoday()
   return 

   ** set the enddate and release the capture, according to users selection      

   function mouse_up
	   this.end_date = this.days[this.pakt].date
      this.switchdate()
   	this.lmd	= false
      ReleaseCapture( )
      this.onenddate()
   return

   ** if the start-date is greater than the end-date, swith them

   function switchdate
      if dtos(this.pdfirst) >= dtos(this.end_date)
         this.starting_date = this.end_date
         this.end_date = this.pdfirst
      else
      	this.starting_date = this.pdfirst
      endif
   return

   ** the following methods get called, when the user makes an action.
   ** they can be seen as placeholders for custom events

   function onStartDate()
   return

   function onEndDate()
   return

   function onchangedate()
   return

   function onAccept()
   return

   function onCancel()
   return

EndClass

** Tdate is the custom object (textlabel) that represents a day.

Class Tdate(oparent,x,y,ni) of textlabel(oparent)
   with (this)
      onLeftMousedown = class::setCLinie
      onmouseover		 = class::mover
      height 	= 22
      left 		= x
      top 		= y
      width 	= 27
      text 		= ""
      colorNormal = normaldatecolor
      fontSize = 12
      fontbold	= true
      borderStyle = 4	// Einfach
      alignHorizontal = 2	// Rechts
   endwith

   this.index  = ni
   this.vcolor = normaldatecolor
   this.date	= new date(1970,1,1)

   function set_vdata
      this.colorNormal = this.vcolor
   return

   ** if the mouse is over a day_textlabel, add it to the selected
   ** days. The method calls the parents method to change the color
   ** of the day

   function mover
   	local pcont
      pcont = this.parent
   	if this.parent.lmd
         pcont.pakt = this.index
         pcont.dcolor()
         pcont.end_date = this.date
         pcont.switchdate()
         pcont.onchangedate()
      endif
   return

   ** if the user clicks on a day, this method sets the data for 
   ** the startdate

   function setCLinie
      local pCont

      ** the startdate must lie in the current month

      if (this.colornormal = normaldatecolor) or (this.colornormal = selectcolor	) or (this.parent.selectable = true)
	      pCont 			= this.parent
   	   pcont.draw(1,42)                // reset all colors
         pcont.settoday()               
      	pcont.pstart	= this.index
	      pcont.pfirst	= this.index
   	   pcont.pakt  	= this.index
	      pcont.starting_date = this.date
   	   pcont.pmfirst	= pcont.make_ym(this.date)
         pcont.pdfirst	= this.date
	      pcont.lmd 		= true    
   	   Setcapture(pcont.hwnd)
      	this.colornormal = selectcolor
	      pcont.onstartdate()
      endif
   return
endclass

** tdaylabel is the custom object for the header

Class Tdaylabel(oparent,x,ct) of textlabel(oparent)
   with (this)
      height 	= 22
      left 		= x
      top 		= 6
      width 	= 27
      text 		= ct
      fontSize = 10
      borderStyle = 3	// Einfach
      alignHorizontal = 1	// Rechts
   endwith
Endclass

