// -- File: METRICS.CC

CLASS Metrics
/*
   ------------------------------------------------------------------
   Description:

      This class contains a set of routines to convert between
      various metrics ... this can be useful if you need to 
      change settings or convert settings between, say, twips
      and inches ...

   Programmers:

    Ken Mayer
 
    History: 

    01/30/1998 -- First version
    01/24/2017 -- Added Points2Pixels and Pixels2Points
 
    Instantiation:
 
    set procedure to metrics.cc additive
    oMyMetrics = new metrics()
 
    Typical usage would be to call individual methods from here,
    such as:
 
      ? oMyMetrics.Twips2Inches( nTwips )
 
    Methods:
       Inches2Twips
       Twips2Inches
       Points2Inches
       Inches2Points
       Twips2Points
       Points2Twips
       CM2Inches
       Inches2CM 
       Points2Pixels
       Pixels2Points
 
    Dependencies:
  
   ------------------------------------------------------------------
*/

   function Inches2Twips( nInches )
   return nInches * 1440

   function Twips2Inches( nTwips )
   return nTwips / 1440

   function Points2Inches( nPoints )
   return nPoints * 72

   function Inches2Points( nInches )
   return nInches / 72

   function Twips2Points( nTwips )
   return nTwips * 20

   function Points2Twips ( nPoints )
   return nPoints / 20

   function CM2Inches( nCM )
   return nCM * 2.54

   function Inches2CM( nInches )
   return nInches / 2.54

   /*
      The following functions are not perfect, but
      may be useful when designing something of a fixed
      height. Proportional fonts in particular can be 
      tricky, with some fonts being taller and/or wider
      than others at the same point size. This does not
      100% take into account descenders (the bottom of
      a lower-case y or p), for example, and it is
      a good idea to add "padding", approx. 4 pixels
      on all sides of a pushbutton, for example (so if
      designing it, add 8 to the height, and 8 to the width).
      If someone finds a way to wrap the .NET
      MeasureString method in dBL, maybe we could get
      something more useful ...? This code was tested
      against:
         https://websemantics.uk/articles/font-size-conversion/
      and works - returning correct values, but dBASE controls 
      don't automatically handle some aspects of the conversion well.
   */
   function Points2Pixels( nPoints )
   return round( nPoints * 1.3333333333333333, 0 )
   
   function Pixels2Points( nPixels )
   return round( nPixels / 1.3333333333333333, 0 )

endclass