//==============================================================================         *
//==============================================================================
//==============================================================================


class Base64 of object


   /* 

	Purpose:    Class for Base64 encoding and decoding of strings. 



   Encode:     Based on code by Bowen Moursund, 2004 01 28

	Decode:     Based on code by Fred Sprokkelenburg, 2008 06 04

   Repackaged: Ronnie MacGregor, 2017 07 21

   				Elements combined into a single class to remove
               outside dependency.

               Test form rmBase64.wfm created.



   Usage

   
   a) Use the class like a common function
   =======================================
   
   set procedure to rmBase64.cc
   oRef = new rmBase64()

   cString = "Sample String" 
   s = oRef.Encode64(cString)
   ? s

   cString="U2FtcGxlIFN0cmluZw==" 
   s = oRef.Decode64(cString)
   ? s


   
   b) Uuse the class like an object with properties and methods
   ============================================================

   set procedure to rmBase64.cc
   oRef = new rmBase64()

   oRef.StringToEncode = "Sample String"
   oRef.Encode64( )
   ? oRef.EncodedString

   obj.StringToDecode = "U2FtcGxlIFN0cmluZw=="
   obj.Decode64()
   ? obj.DecodedString


	*/


   //--------------------------------------------
   //--------------------------------------------
   

   // Initialise object properties

   this.StringToEncode = ""
   this.EncodedString = ""

   this.StringToDecode = ""
   this.DecodedString = ""


   // Initialise Encoding Array

   this.aEncode = new array(64)

   this.aEncode[1]  = "A"
   this.aEncode[2]  = "B"
   this.aEncode[3]  = "C"
   this.aEncode[4]  = "D"
   this.aEncode[5]  = "E"
   this.aEncode[6]  = "F"
   this.aEncode[7]  = "G"
   this.aEncode[8]  = "H"
   this.aEncode[9]  = "I"
   this.aEncode[10] = "J"
   this.aEncode[11] = "K"
   this.aEncode[12] = "L"
   this.aEncode[13] = "M"
   this.aEncode[14] = "N"
   this.aEncode[15] = "O"
   this.aEncode[16] = "P"
   this.aEncode[17] = "Q"
   this.aEncode[18] = "R"
   this.aEncode[19] = "S"
   this.aEncode[20] = "T"
   this.aEncode[21] = "U"
   this.aEncode[22] = "V"
   this.aEncode[23] = "W"
   this.aEncode[24] = "X"
   this.aEncode[25] = "Y"
   this.aEncode[26] = "Z"
   this.aEncode[27] = "a"
   this.aEncode[28] = "b"
   this.aEncode[29] = "c"
   this.aEncode[30] = "d"
   this.aEncode[31] = "e"
   this.aEncode[32] = "f"
   this.aEncode[33] = "g"
   this.aEncode[34] = "h"
   this.aEncode[35] = "i"
   this.aEncode[36] = "j"
   this.aEncode[37] = "k"
   this.aEncode[38] = "l"
   this.aEncode[39] = "m"
   this.aEncode[40] = "n"
   this.aEncode[41] = "o"
   this.aEncode[42] = "p"
   this.aEncode[43] = "q"
   this.aEncode[44] = "r"
   this.aEncode[45] = "s"
   this.aEncode[46] = "t"
   this.aEncode[47] = "u"
   this.aEncode[48] = "v"
   this.aEncode[49] = "w"
   this.aEncode[50] = "x"
   this.aEncode[51] = "y"
   this.aEncode[52] = "z"
   this.aEncode[53] = "0"
   this.aEncode[54] = "1"
   this.aEncode[55] = "2"
   this.aEncode[56] = "3"
   this.aEncode[57] = "4"
   this.aEncode[58] = "5"
   this.aEncode[59] = "6"
   this.aEncode[60] = "7"
   this.aEncode[61] = "8"
   this.aEncode[62] = "9"
   this.aEncode[63] = "+"
   this.aEncode[64] = "/"


   // Initialise Decoding Array

   this.aDecode = new array(128)

   this.aDecode[65]  = 1  // A
   this.aDecode[66]  = 2  // B
   this.aDecode[67]  = 3  // C
   this.aDecode[68]  = 4  // D	
   this.aDecode[69]  = 5  // E
   this.aDecode[70]  = 6  // F
   this.aDecode[71]  = 7  // G
   this.aDecode[72]  = 8  // H
   this.aDecode[73]  = 9  // I
   this.aDecode[74]  = 10 // J
   this.aDecode[75]  = 11 // K
   this.aDecode[76]  = 12 // L
   this.aDecode[77]  = 13 // M
   this.aDecode[78]  = 14 // N
   this.aDecode[79]  = 15 // O
   this.aDecode[80]  = 16 // P
   this.aDecode[81]  = 17 // Q
   this.aDecode[82]  = 18 // R
   this.aDecode[83]  = 19 // S
   this.aDecode[84]  = 20 // T
   this.aDecode[85]  = 21 // U
   this.aDecode[86]  = 22 // V
   this.aDecode[87]  = 23 // W
   this.aDecode[88]  = 24 // X
   this.aDecode[89]  = 25 // Y
   this.aDecode[90]  = 26 // Z
   this.aDecode[97]  = 27 // a
   this.aDecode[98]  = 28 // b
   this.aDecode[99]  = 29 // c
   this.aDecode[100] = 30 // d
   this.aDecode[101] = 31 // e
   this.aDecode[102] = 32 // f
   this.aDecode[103] = 33 // g
   this.aDecode[104] = 34 // h
   this.aDecode[105] = 35 // i
   this.aDecode[106] = 36 // j
   this.aDecode[107] = 37 // k
   this.aDecode[108] = 38 // l
   this.aDecode[109] = 39 // m
   this.aDecode[110] = 40 // n
   this.aDecode[111] = 41 // o
   this.aDecode[112] = 42 // p
   this.aDecode[113] = 43 // q
   this.aDecode[114] = 44 // r
   this.aDecode[115] = 45 // s
   this.aDecode[116] = 46 // t
   this.aDecode[117] = 47 // u
   this.aDecode[118] = 48 // v
   this.aDecode[119] = 49 // w
   this.aDecode[120] = 50 // x
   this.aDecode[121] = 51 // y
   this.aDecode[122] = 52 // z

   this.aDecode[48]  = 53 // 0   
   this.aDecode[49]  = 54 // 1   
   this.aDecode[50]  = 55 // 2   
   this.aDecode[51]  = 56 // 3   
   this.aDecode[52]  = 57 // 4   
   this.aDecode[53]  = 58 // 5   
   this.aDecode[54]  = 59 // 6   
   this.aDecode[55]  = 60 // 7   
   this.aDecode[56]  = 61 // 8   
   this.aDecode[57]  = 62 // 9   
   this.aDecode[43]  = 63 // +   
   this.aDecode[47]  = 64 // /   
   this.aDecode[61]  = 32 


   //--------------------------------------------
   //--------------------------------------------


   function Encode64(cString)

   	/*

    	Method:  Encode64( cStrToEncode )

      Purpose: Returns Base64 encoded argument string

   	*/


      local cStrToEncode, cEncodedStr, i

      if argC() > 0
         cStrToEncode = cString
         this.StringToEncode := cString
      else
         cStrToEncode = this.StringToEncode
      endif

      cEncodedStr = ""

      // Get the characters from the input
      // string in groups of three
      for i = 1 to len( cStrToEncode ) step 3

         // Encode the 3-char group to Base64 and
         // add it to the existing output string
         cEncodedStr += this.Split4( substr( cStrToEncode, i, 3 ) )

      next

      this.EncodedString := cEncodedStr

      return cEncodedStr


   //--------------------------------------------
   //--------------------------------------------


   function Split4(StrToSplit)

	   /*


      Method:  Split4()

      Purpose: Splits a 3 character (24-bit) string into 4 6-bit
               strings, converts them into decimal values and
               returns them as 4 concatenated Base64 characters

      */


      local nOutByte1, nOutByte2, nOutByte3, nOutByte4,;
            nInByte1, nInByte2, nInByte3

      nInByte1 = asc(substr(StrToSplit, 1, 1))
      nInByte2 = asc(substr(StrToSplit, 2, 1))
      nInByte3 = asc(substr(StrToSplit, 3, 1))

      nOutByte1 = bitrshift(nInByte1, 2)
      nOutByte2 = bitlshift(bitand(nInByte1, 0x03), 4) + bitrshift(bitand(nInByte2, 0xf0), 4)
      nOutByte3 = bitlshift(bitand(nInByte2, 0x0f), 2) + bitrshift(bitand(nInByte3, 0xc0), 6)
      nOutByte4 = bitand(nInByte3, 0x3f)

      if len( StrToSplit ) = 3
         return this.aEncode[nOutByte1 + 1] + ;
                this.aEncode[nOutByte2 + 1] + ;
                this.aEncode[nOutByte3 + 1] + ;
                this.aEncode[nOutByte4 + 1]
      elseif len( StrToSplit ) = 2
         return this.aEncode[nOutByte1 + 1] + ;
                this.aEncode[nOutByte2 + 1] + ;
                this.aEncode[nOutByte3 + 1] + ;
                "=" // "=" is specified padding character
      elseif len( StrToSplit ) = 1
         return this.aEncode[nOutByte1 + 1] + ;
                this.aEncode[nOutByte2 + 1] + ;
                "=="
      endif

   return


   //--------------------------------------------
   //--------------------------------------------


   function Decode64(cString)


   	/*

    	Method:  Decode64( cStrToDecode )

      Purpose: Returns Base64 decoded argument string

   	*/


     local cStrToDecode, cDecodedStr, i

      if argC() > 0
         cStrToDecode = cString
         this.StringToDecode := cString
      else
         cStrToDecode = this.StringToDecode
      endif

      cDecodedStr = ""
      // Get the characters from the input
      // string in groups of four

      for i = 1 to len( cStrToDecode ) step 4
         // Decode the 4-char group to Base64 and
         // add it to the existing output string
         cDecodedStr += this.Split3( substr( cStrToDecode, i, 4 ) )
      next

      this.DecodedString := cDecodedStr

      return cDecodedStr


   //--------------------------------------------
   //--------------------------------------------


   function Split3(StrToSplit)

   	/*

      Method:  Split3()

      Purpose: Splits a concatenated Base64 characters (32-bit) string into 3 8-bit
               character strings, converts them into decimal values and
               returns them as 3 asc characters

      */


      local nOutByte1, nOutByte2, nOutByte3,;
            nInByte1, nInByte2, nInByte3, nInByte4

      nInByte1 = asc(substr(StrToSplit, 1, 1))
      nInByte2 = asc(substr(StrToSplit, 2, 1))
      nInByte3 = asc(substr(StrToSplit, 3, 1))
      nInByte4 = asc(substr(StrToSplit, 4, 1))

      nOutByte1 = bitlshift(this.aDecode[nInByte1] - 1 , 2) + bitrshift(bitand(this.aDecode[nInByte2] - 1, 0x30), 4)
      nOutByte2 = bitlshift(bitand(this.aDecode[nInByte2] - 1,  0x0f), 4) + bitand(bitrshift(this.aDecode[nInByte3] - 1 , 2), 0x0f)
      nOutByte3 = bitlshift(bitand(this.aDecode[nInByte3] - 1, 0x03), 6) + bitand(this.aDecode[nInByte4] - 1, 0x3f)
                                   
      If nInByte1 = 61 ; nOutByte1 = 32 ; endif  
      If nInByte2 = 61 ; nOutByte2 = 32 ; endif  
      If nInByte3 = 61 ; nOutByte2 = 32 ; endif  
      If nInByte4 = 61 ; nOutByte3 = 32 ; endif  

      char1 = chr(nOutByte1)
      char2 = chr(nOutByte2)
      char3 = chr(nOutByte3)
      char_return = char1 + char2 + char3

      return  char_return


   //--------------------------------------------
   //--------------------------------------------


 endclass

//==============================================================================
//==============================================================================
//==============================================================================


// EOF /////////////////////////////////////////////////////////////////////////


//==============================================================================
//==============================================================================
//==============================================================================
