/*
    SetCarry.prg

    Programmer...: Ken Mayer
    Date.........: April 2, 2002
    Notes........: This code is the code in the kmCustButt.cc 
                   file for the onClick event of the kmCarryButton. 
                   The idea is to make it available for others 
                   to use without requiring that they use the 
                   custom button class.

                   So, here we go. The code has been removed, and 
                   is being setup as a new program.

                   Note that this is specifically used with OODML
                   code -- you must pass the rowset to copy ...

                   This only works on the current row of a table.
                   If you want to copy multiple rows, you'll have
                   to determine another way of handling this
                   (such as using the updateSet object ...)
    Rev. History.: April 2, 2002 -- Original
    Usage........: do SetCarry with rRowToActOn [, cFieldList]
    Example......: close tables
                   if file( "TestTable.dbf" )
                      drop table TestTable
                   endif

                   create table testtable ( TestAuto autoinc,;
                                            TestChar char(15),;
                                            TestNum  numeric(8,5) )
                   use testtable
                   generate 5
                   use

                   qTest = new query( "select * from TestTable" )
                   qTest.rowset.next() // skip one row
                   // Copy all fields but "TestAuto", which
                   // you can't do:
                   do setCarry with qTest.rowset
                   qTest.rowset.next() // skip one row
                   // only copy the "TestChar" field:
                   do setCarry with qTest.rowset, "TestChar"
                   qTest.active := false
                   // show the contents quickly
                   use TestTable
                   list
                   use

    Parameters...: rRowToActOn = a reference to the rowset
                   cFieldList  = a list of fields to copy,
                                 rather than copying all fields
                                 for the rowset.

    Misc. Notes: In testing, it appears that it's probably not
                 a good idea to use this with an OLE field, so
                 you could put together a field list and skip
                 the OLE field. (I did some testing against the
                 FISH table in the samples folder ...)
*/

parameter rRowToActOn, cFieldList

// check for errors
if type( "rRowToActOn" ) # "O"
   msgbox( "The row object reference is not correct.", "SetCarry: ERROR", 48 )
   return
elseif rRowToActOn.className # "ROWSET"
   msgbox( "The row object reference is not correct.", "SetCarry: ERROR", 48 )
   return
elseif empty( rRowToActOn )
   msgbox( "There is no rowset object to act on.", "SetCarry: ERROR", 48 )
   return
endif

// deal with field list (optional)
if type( "cFieldList" ) == "C"
  lList = true
else
  lList = false
endif

// array used to copy current record
aCarry = new Array()
// shorten the form.rowset.fields references:
fFields = rRowToActOn.fields

// copy current row
for i = 1 to fFields.size
   lAdd = true
   /*
      except for readonly, autoincrement or lock 
      (_dbaseLock) fields 
       NOTE: _dbaseLock fields don't have
            type or readOnly properties,
            so we have to make sure that they
            exist before we check to see what
            is in them ...
   */
   if ( type( "fFields[i].readOnly" ) # "U" and ;
       fFields[i].readOnly ) or ;
         ( type ( "fFields[i].type" ) # "U" and ;
         "AUTOINC" $ fFields[i].type ) or ;
         fFields[i].className == "LOCKFIELD"
      lAdd = false
   endif

   // fields not in cFieldList if provided ...
   if lList AND ;
      NOT upper( fFields[i].fieldName ) $ ;
          upper( cFieldList )
      lAdd = false
   endif

   if not lAdd
      aCarry.add( "* Skip This Field *" )
   else
      /*
         if we have a lookupRowset, then
         we have a lookupSQL statement, and
         we need to ensure that we store the 
         value that needs to be stored, NOT 
         the value that will be returned by 
         the lookupSQL ...
      */
      if fFields[i].lookupRowset # NULL
         aCarry.add( fFields[i].lookupRowset.fields[2].value )
      else
         aCarry.add( fFields[i].value )
      endif
   endif // form.rowset.fields ...
next

// append a new row
rRowToActOn.beginAppend()
            
// -------------------------------
// copy data from previous row:
for i = 1 to fFields.size
   if aCarry[i] == "* Skip This Field *"
      loop
   else
      fFields[i].value := aCarry[i]
   endif  // skip ...
next

rRowToActOn.save()

// redisplay
rRowToActOn.refreshControls()

// let user know:
msgbox( "Row has been copied", "SETCARRY: Complete", 64 )

/*
   End of program: SetCarry.prg
*/
